# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue <https://github.com/pyqrcode/pyqrcodeNG/issues/15>.

Adapted for Segno to check if it suffers from the same problem.
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno


def test_version_too_small():
    with pytest.raises(segno.DataOverflowError) as ex:
        segno.make('A' * 26, version=1)
    assert 'does not fit' in str(ex.value)


def test_version_and_error_provided():
    # QR Code version 1-L: Max. 25 alphanumeric chars
    qr = segno.make('A' * 25, version=1, error='l')
    assert '1-L' == qr.designator
    assert 'alphanumeric' == qr.mode


def test_version_and_error_provided2():
    qr = segno.make('A' * 25, version=1)  # No error level defined
    assert '1-L' == qr.designator
    assert 'alphanumeric' == qr.mode


def test_numeric_defaults():
    qr = segno.make('1' * 17, micro=False)  # Capacity of a 1-H (numeric): 17
    assert '1-H' == qr.designator
    assert 'numeric' == qr.mode


def test_numeric_explicit_error():
    qr = segno.make('1' * 41, error='l')  # Capacity of a 1-L (numeric): 41
    assert '1-L' == qr.designator
    assert 'numeric' == qr.mode


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2021 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the helper factory functions.
Issue <https://github.com/heuer/segno/issues/19>
"""
from __future__ import unicode_literals, absolute_import
import pytest
import segno
from segno import helpers
from datetime import date


def test_geo_default():
    qr = helpers.make_geo(38.8976763, -77.0365297)
    assert not qr.is_micro


def test_geo_data():
    data = helpers.make_geo_data(38.8976763, -77.0365297)
    assert 'geo:38.8976763,-77.0365297' == data


def test_geo_data2():
    data = helpers.make_geo_data(38.89, -77.0365297)
    assert 'geo:38.89,-77.0365297' == data


@pytest.mark.parametrize('expected, ssid, password, security, hidden',
                         (('WIFI:S:SSID;;', 'SSID', None, None, False),
                          ('WIFI:T:SECURITY;S:SSID;;', 'SSID', None, 'security', False),
                          ('WIFI:T:SECURITY;S:SSID;P:secret;;', 'SSID', 'secret', 'security', False),
                          ('WIFI:T:WPA;S:SSID;P:secret;;', 'SSID', 'secret', 'wpa', False),
                          ('WIFI:T:nopass;S:SSID;P:secret;;', 'SSID', 'secret', 'nopass', False),
                          ('WIFI:T:nopass;S:SSID;P:secret;H:true;', 'SSID', 'secret', 'nopass', True),
                          ('WIFI:T:nopass;S:ABCDE;P:abcde;H:true;', 'ABCDE', 'abcde', 'nopass', True),
                          ('WIFI:S:\\"foo\\;bar\\\\baz\\";;', '"foo;bar\\baz"', None, None, False),
                          ('WIFI:T:WPA2;S:\\"foo\\;bar\\\\baz\\";P:a\\:password;;',
                           '"foo;bar\\baz"', 'a:password', 'wpa2', False),
                          ))
def test_wifi_data(expected, ssid, password, security, hidden):
    data = helpers.make_wifi_data(ssid=ssid, password=password, security=security, hidden=hidden)
    assert expected == data


def test_wifi():
    qr = helpers.make_wifi(ssid='SSID', password=None, security='WEP')
    assert qr
    assert not qr.is_micro


def test_mecard_data():
    mecard = helpers.make_mecard_data(name='Mustermann,Max')
    assert 'MECARD:N:Mustermann,Max;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max',
                                      nickname='Maexchen')
    assert 'MECARD:N:Mustermann,Max;NICKNAME:Maexchen;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max', phone=['+1', '+2'])
    assert 'MECARD:N:Mustermann,Max;TEL:+1;TEL:+2;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max',
                                      email='me@example.org')
    assert 'MECARD:N:Mustermann,Max;EMAIL:me@example.org;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max',
                                      birthday=date.today())
    assert 'MECARD:N:Mustermann,Max;BDAY:{0};;'.format(date.today().strftime('%Y%m%d')) == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max', birthday=19760919)
    assert 'MECARD:N:Mustermann,Max;BDAY:19760919;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max', country='Germany')
    assert 'MECARD:N:Mustermann,Max;ADR:,,,,,,Germany;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max',
                                      memo='this,is;a\\memo')
    assert 'MECARD:N:Mustermann,Max;MEMO:this,is\\;a\\\\memo;;' == mecard
    mecard = helpers.make_mecard_data(name='Mustermann,Max',
                                      reading='this,is;a\\sound')
    assert 'MECARD:N:Mustermann,Max;SOUND:this,is\\;a\\\\sound;;' == mecard


def test_mecard():
    qr = helpers.make_mecard(name='Mustermann,Max')
    assert qr


def test_vcard_data():
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Mustermann;Max\r\n' \
           'FN:Max Mustermann\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    org='ABC, Inc.')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Mustermann;Max\r\n' \
           'FN:Max Mustermann\r\n' \
           'ORG:ABC\\, Inc.\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Stevenson;John;Philip,Paul;Dr.;Jr.,M.D.,A.C.P.', 'John Stevenson')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Stevenson;John;Philip,Paul;Dr.;Jr.,M.D.,A.C.P.\r\n' \
           'FN:John Stevenson\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe', street='Street',
                                    city='City', zipcode='123456')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'ADR:;;Street;City;;123456;\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    street='123 Main Street', city='Any Town',
                                    region='CA', zipcode='91921-1234',
                                    country='Nummerland')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'ADR:;;123 Main Street;Any Town;CA;91921-1234;Nummerland\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    title='Python wrangler')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\nFN:John Doe\r\n' \
           'TITLE:Python wrangler\r\n' \
           'END:VCARD\r\n' == vcard
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    title=['Python wrangler', 'Snake charmer'])
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'TITLE:Python wrangler\r\n' \
           'TITLE:Snake charmer\r\n' \
           'END:VCARD\r\n' == vcard
    photo_uri = 'https://www.example.org/image.jpg'
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe', photo_uri=photo_uri)
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'PHOTO;VALUE=uri:{0}\r\n' \
           'END:VCARD\r\n'.format(photo_uri) == vcard
    photo_uris = ('https://www.example.org/image.jpg',
                  'https://www.example.com/image_another.gif')
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    photo_uri=photo_uris)
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'PHOTO;VALUE=uri:{0}\r\n' \
           'PHOTO;VALUE=uri:{1}\r\n' \
           'END:VCARD\r\n'.format(*photo_uris) == vcard


def test_photo_uri():
    photo_uris = ('https://www.example.org/image.jpg',
                  'https://www.example.com/image_another.gif')
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    photo_uri=photo_uris)
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'PHOTO;VALUE=uri:{0}\r\n' \
           'PHOTO;VALUE=uri:{1}\r\n' \
           'END:VCARD\r\n'.format(*photo_uris) == vcard
    qr_from_data = segno.make_qr(vcard)
    assert qr_from_data
    assert qr_from_data.error == 'L'
    qr_from_vcard = helpers.make_vcard('Doe;John', 'John Doe',
                                       photo_uri=photo_uris)
    assert qr_from_vcard
    assert qr_from_vcard.error == 'L'
    assert qr_from_data == qr_from_vcard


def test_vcard_title_escape():
    vcard = helpers.make_vcard_data('Doe;John', 'John Doe',
                                    title='Director, Research and Development')
    assert 'BEGIN:VCARD\r\n' \
           'VERSION:3.0\r\n' \
           'N:Doe;John\r\n' \
           'FN:John Doe\r\n' \
           'TITLE:Director\\, Research and Development\r\n' \
           'END:VCARD\r\n' == vcard


def test_vcard_data_valid_bday():
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'BDAY:1976-09-19\r\n' \
                          'END:VCARD\r\n'
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    birthday='1976-09-19')
    assert expected_vcard_data == vcard
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    birthday=date(year=1976, month=9, day=19))
    assert expected_vcard_data == vcard


def test_vcard_data_invalid_bday():
    with pytest.raises(ValueError):
        helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                birthday='19760919')
    with pytest.raises(ValueError):
        helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                birthday='1976-09-19TZ')


def test_vcard_data_source_url():
    source_url = 'https://example.org/this-is-the-SOURCE-url'
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'SOURCE:{}\r\n' \
                          'END:VCARD\r\n'.format(source_url)
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    source=source_url)
    assert expected_vcard_data == vcard


def test_vcard_data_nickname():
    nickname = 'Mäxchen'
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'NICKNAME:{}\r\n' \
                          'END:VCARD\r\n'.format(nickname)
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    nickname=nickname)
    assert expected_vcard_data == vcard


def test_vcard_data_note():
    note = 'test cases,; we need more test cases'
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'NOTE:test cases\\,\\; we need more test cases\r\n' \
                          'END:VCARD\r\n'
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    memo=note)
    assert expected_vcard_data == vcard


def test_vcard_data_rev():
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'REV:1976-09-19\r\n' \
                          'END:VCARD\r\n'
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    rev='1976-09-19')
    assert expected_vcard_data == vcard
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    rev=date(year=1976, month=9, day=19))
    assert expected_vcard_data == vcard


@pytest.mark.parametrize('rev', ['19760919'
                                 '1976-09-19TZ',
                                 '1976-09-19T-06',
                                 1.2, 1
                                 ])
def test_vcard_data_invalid_rev(rev):
    with pytest.raises(ValueError):
        helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann', rev=rev)


def test_vcard_data_invalid_geo():
    with pytest.raises(ValueError):
        helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann', lat=1.234)
    with pytest.raises(ValueError):
        helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann', lng=1.234)


def test_vcard_data_valid_geo():
    expected_vcard_data = 'BEGIN:VCARD\r\n' \
                          'VERSION:3.0\r\n' \
                          'N:Mustermann;Max\r\n' \
                          'FN:Max Mustermann\r\n' \
                          'GEO:46.235197;8.015445\r\n' \
                          'END:VCARD\r\n'
    vcard = helpers.make_vcard_data('Mustermann;Max', 'Max Mustermann',
                                    lat=46.235197, lng=8.015445)
    assert expected_vcard_data == vcard


def test_vcard():
    qr = helpers.make_vcard(name='Mustermann;Max', displayname='Max Mustermann')
    assert qr


def test_email_data():
    data = helpers.make_make_email_data('me@example.org')
    assert 'mailto:me@example.org' == data
    data = helpers.make_make_email_data(('me@example.org', 'you@example.org'))
    assert 'mailto:me@example.org,you@example.org' == data
    data = helpers.make_make_email_data('me@example.org', cc='you@example.org')
    assert 'mailto:me@example.org?cc=you@example.org' == data
    data = helpers.make_make_email_data('me@example.org', bcc='you@example.org')
    assert 'mailto:me@example.org?bcc=you@example.org' == data
    data = helpers.make_make_email_data('me@example.org', cc=('me@example.org',
                                                              'you@example.org'))
    assert 'mailto:me@example.org?cc=me@example.org,you@example.org' == data
    data = helpers.make_make_email_data('me@example.org', bcc=('me@example.org',
                                                               'you@example.org'))
    assert 'mailto:me@example.org?bcc=me@example.org,you@example.org' == data
    data = helpers.make_make_email_data('me@example.org', cc=('me@example.org',
                                                              'you@example.org'),
                                        subject='Test')
    assert 'mailto:me@example.org?cc=me@example.org,you@example.org&subject=Test' == data
    data = helpers.make_make_email_data('me@example.org', cc=('me@example.org',
                                                              'you@example.org'),
                                        subject='Subject', body='Body')
    assert 'mailto:me@example.org?cc=me@example.org,you@example.org&subject=Subject&body=Body' == data
    data = helpers.make_make_email_data('me@example.org', subject='A subject',
                                        body='Hellöö')
    assert 'mailto:me@example.org?subject=A%20subject&body=Hell%C3%B6%C3%B6' == data


def test_email_data_illegal():
    with pytest.raises(ValueError):
        helpers.make_make_email_data(None)
    with pytest.raises(ValueError):
        helpers.make_make_email_data('')
    with pytest.raises(ValueError):
        helpers.make_make_email_data([])


def test_email():
    qr = helpers.make_email('me@example.org')
    assert qr


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 39
<https://github.com/heuer/segno/issues/39>
"""
from __future__ import absolute_import, unicode_literals
import os
import io
import tempfile
import pytest
import segno
from segno import cli


def test_output():
    out = io.BytesIO()
    segno.make_qr('Good Times', error='M').save(out, kind='png', scale=10,
                                                dark='red')
    f = tempfile.NamedTemporaryFile('w', suffix='.png', delete=False)
    f.close()
    cli.main(['-e=M', '--scale=10', '--dark=red', '--output={0}'.format(f.name),
              'Good Times'])
    f = open(f.name, 'rb')
    content = f.read()
    f.close()
    os.unlink(f.name)
    assert out.getvalue() == content


def test_output2():
    out = io.BytesIO()
    segno.make_qr('Good Times', error='M').save(out, kind='png', scale=10,
                                                dark='red')
    f = tempfile.NamedTemporaryFile('w', suffix='.png', delete=False)
    f.close()
    cli.main(['-e=M', '--scale=10', '--dark=red', '--output={0}'.format(f.name),
              'Good', 'Times'])
    f = open(f.name, 'rb')
    content = f.read()
    f.close()
    os.unlink(f.name)
    assert out.getvalue() == content


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test suite which decodes generated QR Codes.

Requires pyzbar and additional libs (libzbar0).
"""
from __future__ import absolute_import, unicode_literals
import io
import pytest
import segno
try:
    FileNotFoundError
except NameError:  # Py2
    FileNotFoundError = OSError
try:
    from pyzbar.pyzbar import decode as zbardecode
except (ImportError, FileNotFoundError):  # The latter may occur under Windows
    pytestmark = pytest.mark.skip


def qr_to_bytes(qrcode, scale):
    if qrcode.is_micro:
        raise Exception('zbar cannot decode Micro QR codes')
    buff = io.BytesIO()
    for row in qrcode.matrix_iter(scale=scale):
        buff.write(bytearray(0x0 if b else 0xff for b in row))
    return buff.getvalue()


def decode(qrcode):
    scale = 3
    width, height = qrcode.symbol_size(scale=scale)
    qr_bytes = qr_to_bytes(qrcode, scale)
    decoded = zbardecode((qr_bytes, width, height))
    assert 1 == len(decoded)
    assert 'QRCODE' == decoded[0].type
    return decoded[0].data.decode('utf-8')


@pytest.mark.parametrize('content, mode',
                         [('漢字', 'kanji'),
                          ('続きを読む', 'kanji'),
                          ('Märchenbücher', 'byte'),
                          ('汉字', 'byte'),
                          ])
def test_encode_decode(content, mode):
    qr = segno.make_qr(content)
    assert mode == qr.mode
    assert content == decode(qr)


def test_stackoverflow_issue():
    # See <https://stackoverflow.com/questions/63303624/generating-and-reading-qr-codes-with-special-characters>
    # and <https://github.com/NaturalHistoryMuseum/pyzbar/issues/14>
    content = 'Thomsôn Gonçalves Ámaral,325.432.123-21'
    qr = segno.make(content, encoding='utf-8')
    assert 'byte' == qr.mode
    assert content == decode(qr).encode('shift-jis').decode('utf-8')


def test_pyqrcode_issue76():
    # See <https://github.com/mnooner256/pyqrcode/issues/76>
    content = 'АБВГД'
    qr = segno.make(content, micro=False)
    assert 'kanji' == qr.mode
    assert content == decode(qr)
    qr = segno.make(content, encoding='utf-8', micro=False)
    assert 'byte' == qr.mode
    assert content == decode(qr)
    qr = segno.make(content.encode('utf-8'), micro=False)
    assert 'byte' == qr.mode
    assert content == decode(qr)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #54.
<https://github.com/heuer/segno/issues/54>
"""
from __future__ import unicode_literals, absolute_import
import io
import pytest
from png import Reader as PNGReader
import segno


@pytest.mark.parametrize('dark, light, transparent', [('white', None, (0,)),
                                                      ('white', 'black', None),
                                                      (None, 'black', (1,))])
def test_issue_54_inverted(dark, light, transparent):
    qr = segno.make('The Beatles')
    assert 'M4-M' == qr.designator
    out = io.BytesIO()
    scale = 5
    qr.save(out, kind='png', scale=scale, dark=dark, light=light)
    out.seek(0)
    reader = PNGReader(file=out)
    w, h, pixels, meta = reader.read()
    width, height = qr.symbol_size(scale=scale)
    assert width == w
    assert height == h
    assert meta['greyscale']
    border_row = tuple([0] * w)
    expected_row = tuple([0] * 2 * scale + [1] * 7 * scale + [0])
    for idx, row in enumerate(pixels):
        if idx < 10:
            assert border_row == tuple(row)
        elif idx == 10:
            assert expected_row == tuple(row)[:len(expected_row)]
            break
    assert transparent == reader.transparent


@pytest.mark.parametrize('dark, light, transparent', [('black', None, (1,)),
                                                      ('black', 'white', None),
                                                      (None, 'white', (0,))])
def test_issue_54_notinverted(dark, light, transparent):
    qr = segno.make('The Beatles')
    assert 'M4-M' == qr.designator
    out = io.BytesIO()
    scale = 5
    qr.save(out, kind='png', scale=scale, dark=dark, light=light)
    out.seek(0)
    reader = PNGReader(file=out)
    w, h, pixels, meta = reader.read()
    width, height = qr.symbol_size(scale=scale)
    assert width == w
    assert height == h
    assert meta['greyscale']
    border_row = tuple([1] * w)
    expected_row = tuple([1] * 2 * scale + [0] * 7 * scale + [1])
    for idx, row in enumerate(pixels):
        if idx < 10:
            assert border_row == tuple(row)
        elif idx == 10:
            assert expected_row == tuple(row)[:len(expected_row)]
            break
    assert transparent == reader.transparent


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue 33 and 37
<https://github.com/heuer/segno/issues/33>
<https://github.com/heuer/segno/issues/37>
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno
from segno import consts
from segno import encoder
try:
    from .tutils import read_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import read_matrix


def test_format_info_figure25():
    # 7.9.1 QR Code symbols (page 55)
    version = 1
    mask_pattern = 5
    error = consts.ERROR_LEVEL_M
    # 100000011001110
    assert 0x40ce == encoder.calc_format_info(version, error, mask_pattern)


def test_format_info_figure26():
    # 7.9.2 Micro QR Code symbols (page 57)
    version = consts.VERSION_M1
    mask_pattern = 3
    error = None
    # 100101100011100
    assert 0x4b1c == encoder.calc_format_info(version, error, mask_pattern)


@pytest.mark.parametrize('data', ['50041', 50041])
@pytest.mark.parametrize('version', [None, 'm1'])
def test_m1_50041(data, version):
    qr = segno.make(data, version=version)
    assert 'M1' == qr.designator
    ref_matrix = read_matrix('issue-33-m1-50041')
    assert ref_matrix == qr.matrix


@pytest.mark.parametrize('data', ['1234', 1234])
@pytest.mark.parametrize('version', [None, 'm1'])
def test_m1_1234(data, version):
    qr = segno.make(data, version=version)
    assert 'M1' == qr.designator
    ref_matrix = read_matrix('issue-33-m1-1234')
    assert ref_matrix == qr.matrix


@pytest.mark.parametrize('data', ['12345', 12345])
@pytest.mark.parametrize('version', [None, 'm1'])
def test_m1_12345(data, version):
    qr = segno.make(data, version=version)
    assert 'M1' == qr.designator
    ref_matrix = read_matrix('issue-33-m1-12345')
    assert ref_matrix == qr.matrix


def test_m3_wikipedia():
    qr = segno.make('Wikipedia', version='m3', error='l')
    assert 'M3-L' == qr.designator
    ref_matrix = read_matrix('issue-33-m3-l-wikipedia')
    assert ref_matrix == qr.matrix


def test_m3_max_numeric():
    qr = segno.make('12345678901234567890123', version='m3', error='l')
    assert 'M3-L' == qr.designator
    ref_matrix = read_matrix('issue-33-m3-l-12345678901234567890123')
    assert ref_matrix == qr.matrix


def test_jump_from_m3_to_m4_dont_boost_error():
    qr = segno.make('123456789012345678901234', boost_error=False)
    assert 'M4-L' == qr.designator
    ref_matrix = read_matrix('issue-33-m3-l-to-m4-l-jump')
    assert ref_matrix == qr.matrix


def test_jump_from_m3_to_m4_boost_error():
    qr = segno.make('123456789012345678901234')
    assert 'M4-M' == qr.designator
    ref_matrix = read_matrix('issue-33-m3-l-to-m4-m-jump')
    assert ref_matrix == qr.matrix


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
EPC QR Codes.

Test against issue <https://github.com/heuer/segno/issues/55>.
"""
from __future__ import absolute_import, unicode_literals
import decimal
import pytest
from segno.helpers import make_epc_qr, _make_epc_qr_data as make_epc_qr_data


@pytest.mark.parametrize('amount', [12.3,
                                    12.30,
                                    decimal.Decimal('12.3'),
                                    decimal.Decimal('12.30'),
                                    '12.3',
                                    '12.30'])
def test_text_002(amount):
    name = "François D'Alsace S.A."
    iban = 'FR1420041010050500013M02606'
    text = 'Client:Marie Louise La Lune'
    kw = dict(name=name, iban=iban, text=text, amount=amount)
    data = make_epc_qr_data(**kw)
    # See. EPC069-12 Version 2.1 dtd. 9 February 2012 example 2
    assert len(data) == 103
    encoding = 'iso-8859-1'
    d = [x.decode(encoding) for x in data.split(b'\n')]
    assert 11 == len(d)
    assert 'BCD' == d[0]
    assert '002' == d[1]
    assert '2' == d[2]
    assert 'SCT' == d[3]
    assert name == d[5]
    assert iban == d[6]
    assert 'EUR12.3' == d[7]
    assert '' == d[8]
    assert '' == d[9]
    assert text == d[10]
    qr = make_epc_qr(**kw)
    assert qr
    assert not qr.is_micro
    assert qr.version <= 13
    assert 'M' == qr.error


@pytest.mark.parametrize('expected_amount, amount', [('EUR1000', 1000),
                                                     ('EUR1000', 1000.0),
                                                     ('EUR2000', decimal.Decimal('2000'))])
def test_trailing_zeros(expected_amount, amount):
    name = "François D'Alsace S.A."
    iban = 'FR1420041010050500013M02606'
    text = 'Client:Marie Louise La Lune'
    kw = dict(name=name, iban=iban, text=text, amount=amount)
    data = make_epc_qr_data(**kw)
    assert len(data) == 103  # See. EPC069-12 Version 2.1 dtd. 9 February 2012 example 2
    encoding = 'iso-8859-1'
    d = [x.decode(encoding) for x in data.split(b'\n')]
    assert expected_amount == d[7]


@pytest.mark.parametrize('amount', [5.0, 5, '5.00', decimal.Decimal('5.00000')])
def test_remove_dot(amount):
    kw = _make_valid_kw()
    kw['amount'] = amount
    d = make_epc_qr_data(**kw).split(b'\n')
    assert b'EUR5' == d[7]


@pytest.mark.parametrize('amount', [12.3,
                                    12.30,
                                    decimal.Decimal('12.3'),
                                    decimal.Decimal('12.30'),
                                    '12.3',
                                    '12.30'])
def test_reference_002(amount):
    name = 'Franz Mustermänn'
    iban = 'DE71110220330123456789'
    reference = 'RF18539007547034'
    purpose = 'GDDS'
    bic = 'BHBLDEHHXXX'
    kw = dict(name=name,
              iban=iban,
              reference=reference,
              bic=bic,
              purpose=purpose,
              amount=amount,
              encoding=1)
    data = make_epc_qr_data(**kw)
    assert len(data) == 96  # See. EPC069-12 Version 2.1 dtd. 9 February 2012 example 1
    encoding = 'utf-8'
    d = [x.decode(encoding) for x in data.split(b'\n')]
    assert 10 == len(d)
    assert 'BCD' == d[0]
    assert '002' == d[1]
    assert '1' == d[2]
    assert 'SCT' == d[3]
    assert name == d[5]
    assert iban == d[6]
    assert 'EUR12.3' == d[7]
    assert purpose == d[8]
    assert reference == d[9]
    qr = make_epc_qr(**kw)
    assert qr
    assert not qr.is_micro
    assert qr.version <= 13
    assert 'M' == qr.error


def _make_valid_kw():
    return dict(name="François D'Alsace S.A.",
                iban='FR1420041010050500013M02606',
                text='Client:Marie Louise La Lune',
                amount=12.3)


@pytest.mark.parametrize('amount', [0,
                                    0.004,
                                    '0.001',
                                    '999999999.999',
                                    9999999990.99])
def test_invalid_amount(amount):
    kw = _make_valid_kw()
    kw['amount'] = amount
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'amount' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'amount' in str(ex.value)


@pytest.mark.parametrize('bic', ['BHBLDE',  # Too short
                                 'BHBLDEHHXXXX',  # Too long
                                 'BHBLDEHHXX',  # Too short (either 8 or 11) not 8 <= bic <= 11
                                 'BHBLDEH ',  # Too short after removing trailing WS
                                 ])
def test_invalid_bic(bic):
    kw = _make_valid_kw()
    kw['bic'] = bic
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'BIC' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'BIC' in str(ex.value)


def test_utf8_required():
    kw = _make_valid_kw()
    kw['name'] = 'Funny 😃 name'
    d = make_epc_qr_data(**kw).split(b'\n')
    assert b'1' == d[2]


def test_utf8_explicit():
    kw = _make_valid_kw()
    kw['encoding'] = 'utf-8'
    kw['name'] = 'Funny 😃 name'
    d = make_epc_qr_data(**kw).split(b'\n')
    assert b'1' == d[2]


def test_utf8_explicit2():
    kw = _make_valid_kw()
    kw['encoding'] = 1
    kw['name'] = 'Funny 😃 name'
    d = make_epc_qr_data(**kw).split(b'\n')
    assert b'1' == d[2]


@pytest.mark.parametrize('encoding', range(1, 9))
def test_valid_encoding(encoding):
    kw = _make_valid_kw()
    kw['name'] = 'Simple name'
    kw['encoding'] = encoding
    d = make_epc_qr_data(**kw).split(b'\n')
    assert str(encoding).encode() == d[2]
    qr = make_epc_qr(**kw)
    assert qr


@pytest.mark.parametrize('encoding', [0, 9, '1', b'8', 1.0, 'shift-jis'])
def test_illegal_encoding(encoding):
    kw = _make_valid_kw()
    kw['encoding'] = encoding
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'encoding' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'encoding' in str(ex.value)


@pytest.mark.parametrize('text,reference', [('', ''), (' ', '    '),
                                            ('', None), (None, None),
                                            (None, ' '),
                                            ])
def test_no_text_no_reference(text, reference):
    kw = _make_valid_kw()
    kw['text'] = text
    kw['reference'] = reference
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'reference' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'reference' in str(ex.value)


@pytest.mark.parametrize('iban', ['DE1' + '1' * 34,
                                  '',
                                  None])
def test_illegal_iban(iban):
    kw = _make_valid_kw()
    kw['iban'] = iban
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'IBAN' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'IBAN' in str(ex.value)


@pytest.mark.parametrize('purpose', ['DE1', 'x', 'CDCBC'])
def test_illegal_purpose(purpose):
    kw = _make_valid_kw()
    kw['purpose'] = purpose
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'purpose' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'purpose' in str(ex.value)


@pytest.mark.parametrize('name', [None, '',
                                  'a' * 71,  # too long
                                  ])
def test_illegal_name(name):
    kw = _make_valid_kw()
    kw['name'] = name
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'name' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        make_epc_qr(**kw)
    assert 'name' in str(ex.value)


def test_text_too_long():
    kw = _make_valid_kw()
    kw['text'] = 'a' * 141
    kw['reference'] = None
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'text' in str(ex.value)


def test_reference_too_long():
    kw = _make_valid_kw()
    kw['text'] = None
    kw['reference'] = 'r' * 36
    with pytest.raises(ValueError) as ex:
        make_epc_qr_data(**kw)
    assert 'reference' in str(ex.value)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
QR Code tests.
"""
from __future__ import absolute_import, unicode_literals
import os
import io
from itertools import chain
import gzip
import tempfile
import pytest
import segno
from segno import consts
try:  # Py 2
    unicode
    from io import open
except NameError:
    pass

_LEGAL_MICRO_VERSIONS = tuple(chain(consts.MICRO_VERSION_MAPPING.keys(),
                                    [v.lower() for v in consts.MICRO_VERSION_MAPPING.keys()]))
_LEGAL_VERSIONS = tuple(chain(range(1, 41), [str(v) for v in range(1, 41)]))

_LEGAL_ERROR_LEVELS = tuple(chain(consts.ERROR_MAPPING.keys(),
                                  [e.lower() for e in consts.ERROR_MAPPING.keys()]))


def test_eq():
    qr = segno.make('Equals')
    assert qr != qr.matrix


def test_eq2():
    qr = segno.make('Equals')
    qr2 = segno.make('Equals')
    assert qr == qr2


@pytest.mark.parametrize('version, mode', [('M1', 'alphanumeric'),
                                           ('M1', 'byte'),
                                           ('M2', 'byte')])
def test_illegal_mode_micro(version, mode):
    with pytest.raises(ValueError) as ex:
        segno.make(1, version=version, mode=mode)
    assert 'is not available' in str(ex.value)
    with pytest.raises(ValueError) as ex:
        segno.make(1, version=version.lower(), mode=mode)


@pytest.mark.parametrize('version', _LEGAL_MICRO_VERSIONS)
def test_micro_version_contradicts_micro(version):
    with pytest.raises(ValueError) as ex:
        segno.make(1, version=version, micro=False)
    assert '"micro" is False' in str(ex.value)


@pytest.mark.parametrize('version', _LEGAL_VERSIONS)
def test_version_contradicts_micro(version):
    with pytest.raises(ValueError) as ex:
        segno.make(1, version=version, micro=True)
    assert 'version' in str(ex.value)


@pytest.mark.parametrize('version', ['M0', 'M5', -1, 0, 41, 'M1 ', object()])
def test_illegal_version(version):
    with pytest.raises(ValueError) as ex:
        segno.make('a', version=version)
    assert 'version' in str(ex.value)


@pytest.mark.parametrize('version', _LEGAL_MICRO_VERSIONS)
def test_valid_mirco_versions(version):
    qr = segno.make(1, version=version)
    assert version.upper() == qr.version
    assert qr.is_micro


@pytest.mark.parametrize('version', _LEGAL_VERSIONS)
def test_valid_versions(version):
    qr = segno.make(1, version=version)
    assert int(version) == qr.version
    assert not qr.is_micro


@pytest.mark.parametrize('error', _LEGAL_ERROR_LEVELS)
def test_legal_error_levels(error):
    qr = segno.make(1, error=error, boost_error=False)
    assert error.upper() == qr.error


@pytest.mark.parametrize('error', ['R', 'M ', ' L'])
def test_illegal_error_level(error):
    with pytest.raises(ValueError) as ex:
        segno.make(1, error=error)
    assert 'illegal error correction' in str(ex.value).lower()
    assert 'L, M, Q, H' in str(ex.value)


def test_illegal_error_level_micro():
    with pytest.raises(ValueError) as ex:
        segno.make('test', error='H', micro=True)
    assert 'is not available' in str(ex.value)


@pytest.mark.parametrize('data,version', [('abcdefghijklmnopqr', 1),
                                          (123456, 'M1')])
def test_data_too_large(data, version):
    with pytest.raises(segno.DataOverflowError) as ex:
        segno.make(data, version=version)
    assert 'does not fit' in str(ex.value)


def test_eci_and_micro():
    with pytest.raises(ValueError) as ex:
        segno.make('A', eci=True, micro=True)
    assert 'ECI mode' in str(ex.value)


def test_eci_and_micro2():
    with pytest.raises(ValueError) as ex:
        segno.make('A', eci=True, version='m4')
    assert 'ECI mode' in str(ex.value)


def _calc_size(dim, border, scale=1):
    return (dim + 2 * border) * scale


def test_symbol_size():
    qr = segno.make('Hello world', micro=False)
    width, height = 21, 21
    border = 0
    assert (width, height) == qr.symbol_size(border=border)
    border = 1
    assert (_calc_size(width, border) == _calc_size(width, border)), \
        qr.symbol_size(border=border)
    border = 4  # (default border)
    assert (_calc_size(width, border) == _calc_size(width, border)), \
        qr.symbol_size()
    assert (_calc_size(width, border) == _calc_size(width, border)), \
        qr.symbol_size(scale=1)
    assert (_calc_size(width, border, 4), _calc_size(width, border, 4)) == \
           qr.symbol_size(scale=4)
    border = 0
    assert (_calc_size(width, border, 4), _calc_size(width, border, 4)) == \
           qr.symbol_size(border=border, scale=4)


def test_symbol_size_micro():
    qr = segno.make('A', version='m2')
    width, height = 13, 13  # Micro QR Code Version M2 size
    border = 0
    assert (width, height) == qr.symbol_size(border=border)
    border = 1
    dim = _calc_size(width, border)
    assert (dim, dim) == qr.symbol_size(border=border)
    border = 2  # (default border)
    dim = _calc_size(width, border)
    assert (dim, dim) == qr.symbol_size()
    assert (dim, dim) == qr.symbol_size(scale=1)
    dim = _calc_size(width, border, 4)
    assert (dim, dim) == qr.symbol_size(scale=4)
    border = 0
    dim = _calc_size(width, border, 4)
    assert (dim, dim) == qr.symbol_size(border=border, scale=4)


def test_symbol_size_scale_int():
    qr = segno.make_qr('test')
    assert (21, 21) == qr.symbol_size(border=0)


def test_symbol_size_scale_int2():
    qr = segno.make_qr('test')
    border = 2
    dim = 21 + 2 * border
    assert (dim, dim) == qr.symbol_size(border=border)


def test_symbol_size_scale_float():
    qr = segno.make_qr('test')
    dim = 21 * 2.5
    assert (dim, dim) == qr.symbol_size(scale=2.5, border=0)


def test_symbol_size_scale_float2():
    qr = segno.make_qr('test')
    border = 2
    dim = (21 + 2 * border) * 1.5
    assert (dim, dim) == qr.symbol_size(scale=1.5, border=border)


def test_designator():
    qr = segno.make('test', version=40, error='L', boost_error=False)
    assert '40-L' == qr.designator


def test_designator2():
    qr = segno.make('test', version=8, error='m', boost_error=False)
    assert '8-M' == qr.designator


def test_designator_micro():
    qr = segno.make('test', version='M4', error='L', boost_error=False)
    assert 'M4-L' == qr.designator


def test_designator_micro2():
    qr = segno.make('12', version='M1')
    assert 'M1' == qr.designator


def test_error_m1():
    qr = segno.make('12')
    assert 'M1' == qr.version
    assert qr.error is None


def test_default_border():
    qr = segno.make_qr(12)
    assert 4 == qr.default_border_size


def test_default_border_mirco():
    qr = segno.make_micro(12, version='m4')
    assert 2 == qr.default_border_size


def test_neq():
    qr = segno.make('Hello')
    qr2 = segno.make('hello')
    assert qr == qr
    assert qr2 == qr2
    assert qr != qr2


@pytest.mark.parametrize('border', [.2, -1, 1.3])
def test_matrix_iter_invalid_border(border):
    qr = segno.make('A')
    with pytest.raises(ValueError):
        for row in qr.matrix_iter(border=border):
            pass


def test_matrix_iter_border_zero():
    qr = segno.make('No border')
    res = [bytearray(row) for row in qr.matrix_iter(border=0)]
    assert qr.matrix == tuple(res)


def test_matrix_iter_border_default():
    qr = segno.make('A', version=1)
    res = [bytearray(row) for row in qr.matrix_iter(border=None)]
    top_border = [bytearray([0x0] * 29)] * 4
    seq = bytearray([0x0, 0x0, 0x0, 0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0])
    assert top_border == res[:4]
    assert seq == res[4][:len(seq)]


def test_matrix_iter_border_3():
    qr = segno.make('A', version=1)
    res = [bytearray(row) for row in qr.matrix_iter(border=3)]
    top_border = [bytearray([0x0] * 27)] * 3
    seq = bytearray([0x0, 0x0, 0x0, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x0])
    assert top_border == res[:3]
    assert seq == res[3][:len(seq)]


@pytest.mark.parametrize('border', [.2, -1, 1.3])
def test_matrix_iter_verbose_invalid_border(border):
    qr = segno.make('A')
    with pytest.raises(ValueError):
        for row in qr.matrix_iter(border=border, verbose=True):
            pass


def test_matrix_iter_verbose_border_zero():
    qr = segno.make('No border')
    res = [bytearray([bool(v >> 8) for v in row]) for row in qr.matrix_iter(
        border=0,
        verbose=True)]
    assert qr.matrix == tuple(res)


def test_matrix_iter_verbose_border_default():
    qr = segno.make('A', version=1)
    res = [list(row) for row in qr.matrix_iter(border=None, verbose=True)]
    top_border = [[consts.TYPE_QUIET_ZONE] * 29] * 4
    seq = []
    seq.extend([consts.TYPE_QUIET_ZONE] * 4)
    seq.extend([consts.TYPE_FINDER_PATTERN_DARK] * 7)
    seq.extend([consts.TYPE_SEPARATOR, consts.TYPE_FORMAT_LIGHT])
    assert top_border == res[:4]
    assert seq == res[4][:len(seq)]


def test_matrix_iter_verbose_border_3():
    qr = segno.make('A', version=1)
    res = [list(row) for row in qr.matrix_iter(border=3, verbose=True)]
    top_border = [[consts.TYPE_QUIET_ZONE] * 27] * 3
    seq = []
    seq.extend([consts.TYPE_QUIET_ZONE] * 3)
    seq.extend([consts.TYPE_FINDER_PATTERN_DARK] * 7)
    seq.extend([consts.TYPE_SEPARATOR, consts.TYPE_FORMAT_LIGHT])
    assert top_border == res[:3]
    assert seq == res[3][:len(seq)]


def test_save_png_buffer():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='png')
    out.seek(0)
    expected = b'\211PNG\r\n\032\n'  # PNG magic number
    val = out.read(len(expected))
    assert expected == val


def test_save_png_filestream():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.png', delete=False) as f:
        fn = f.name
        qr.save(f)
    expected = b'\211PNG\r\n\032\n'  # PNG magic number
    with open(fn, mode='rb') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_png_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.png', delete=False) as f:
        fn = f.name
    qr.save(fn)
    expected = b'\211PNG\r\n\032\n'  # PNG magic number
    with open(fn, mode='rb') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


@pytest.mark.parametrize('ext', ['svg', 'SvG', 'SVG', 'Svg'])
def test_save_svg_filestream(ext):
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.' + ext, delete=False) as f:
        fn = f.name
        qr.save(f)
    with open(fn, mode='rb') as f:
        val = f.read(6)
    os.unlink(fn)
    assert b'<?xml ' == val


def test_save_svg_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.svg', delete=False) as f:
        fn = f.name
        qr.save(f.name)
    with open(fn, mode='rb') as f:
        val = f.read(6)
    os.unlink(fn)
    assert b'<?xml ' == val


def test_save_svgz_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.svgz', delete=False) as f:
        fn = f.name
    qr.save(fn)
    expected = b'\x1f\x8b\x08'  # gzip magic number
    with open(fn, mode='rb') as f:
        val = f.read(len(expected))
    with gzip.open(fn) as f:
        content = f.read(6)
    os.unlink(fn)
    assert expected == val
    assert b'<?xml ' == content


def test_save_pdf_filestream():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.pdf', delete=False) as f:
        fn = f.name
        qr.save(f)
    with open(fn, mode='rb') as f:
        val = f.read(5)
    os.unlink(fn)
    assert b'%PDF-' == val


def test_save_pdf_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('wb', suffix='.pdf', delete=False) as f:
        fn = f.name
        qr.save(fn)
    with open(fn, mode='rb') as f:
        val = f.read(5)
    os.unlink(fn)
    assert b'%PDF-' == val


def test_save_eps_filestream():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.eps', delete=False) as f:
        fn = f.name
        qr.save(f)
    expected = '%!PS-Adobe-3.0 EPSF-3.0'
    with open(fn, mode='r') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_eps_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.eps', delete=False) as f:
        fn = f.name
    qr.save(fn)
    expected = '%!PS-Adobe-3.0 EPSF-3.0'
    with open(fn, mode='r') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_txt_filestream():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.txt', delete=False) as f:
        fn = f.name
        qr.save(f)
    expected = '000000'
    with open(fn, mode='r') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_txt_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.txt', delete=False) as f:
        fn = f.name
        qr.save(fn)
    expected = '000000'
    with open(fn, mode='r', encoding='utf-8') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


@pytest.mark.parametrize('kind', ['eps', 'EpS', 'EPS', 'Eps'])
def test_save_kind_filestream(kind):
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.murks', delete=False) as f:
        fn = f.name
        qr.save(f, kind=kind)
    expected = '%!PS-Adobe-3.0 EPSF-3.0'
    with open(fn, mode='r') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_kind_filename():
    qr = segno.make_qr('test')
    f = tempfile.NamedTemporaryFile('w', suffix='.murks', delete=False)
    f.close()
    qr.save(f.name, kind='eps')
    f = open(f.name, mode='r')
    expected = '%!PS-Adobe-3.0 EPSF-3.0'
    val = f.read(len(expected))
    f.close()
    os.unlink(f.name)
    assert expected == val


def test_save_kind_overrides_filename():
    qr = segno.make_qr('test')
    # SVG extension
    with tempfile.NamedTemporaryFile('w', suffix='.svg', delete=False) as f:
        fn = f.name
    # ... but we want EPS
    qr.save(fn, kind='eps')
    expected = '%!PS-Adobe-3.0 EPSF-3.0'
    with open(f.name, mode='r') as f:
        val = f.read(len(expected))
    os.unlink(fn)
    assert expected == val


def test_save_invalid_filename():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.murks', delete=True) as f:
        fn = f.name
    with pytest.raises(ValueError):
        qr.save(fn)


def test_save_invalid_filename2():
    qr = segno.make_qr('test')
    with tempfile.NamedTemporaryFile('w', suffix='.murks', delete=True) as f:
        with pytest.raises(ValueError):
            qr.save(f)


def test_unknown_converter():
    qr = segno.make_qr('test')
    with pytest.raises(AttributeError):
        qr.to_murks()


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
EPS related tests.
"""
from __future__ import absolute_import, unicode_literals
import re
import io
import pytest
import segno


def test_illegal_color_float():
    color = (.1, 1.1, .1)
    qr = segno.make_qr('test')
    out = io.StringIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='eps', dark=color)


def test_illegal_color_float2():
    color = (-.1, 1.0, .1)
    qr = segno.make_qr('test')
    out = io.StringIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='eps', dark=color)


def test_illegal_color_int():
    color = (255, 255, 256)
    qr = segno.make_qr('test')
    out = io.StringIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='eps', dark=color)


def test_illegal_color_int2():
    color = (-1, 1, 1)
    qr = segno.make_qr('test')
    out = io.StringIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='eps', dark=color)


def test_default_color():
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.save(out, kind='eps')
    assert 'setrgbcolor' not in out.getvalue()


def test_color():
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.save(out, kind='eps', dark='#195805')
    assert 'setrgbcolor' in out.getvalue()


def test_color_omit_black():
    qr = segno.make_qr('test')
    out = io.StringIO()
    # Black does not need setrgbcolor since it is the default stroke color
    qr.save(out, kind='eps', dark='#000')
    assert 'setrgbcolor' not in out.getvalue()


def test_color_omit_black2():
    qr = segno.make_qr('test')
    out = io.StringIO()
    # Black does not need setrgbcolor since it is the default stroke color
    qr.save(out, kind='eps', dark='Black')
    assert 'setrgbcolor' not in out.getvalue()


def test_color_omit_black3():
    qr = segno.make_qr('test')
    out = io.StringIO()
    # Black does not need setrgbcolor since it is the default stroke color
    qr.save(out, kind='eps', dark=(0, 0, 0))
    assert 'setrgbcolor' not in out.getvalue()


def test_background():
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.save(out, kind='eps', light='#EEE')
    assert 'setrgbcolor' in out.getvalue()
    assert 'clippath' in out.getvalue()


def test_default_scale():
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.save(out, kind='eps')
    assert 'scale' not in out.getvalue()


def test_scale():
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.save(out, kind='eps', scale=2)
    assert '2 2 scale' in out.getvalue()


def test_scale_float():
    qr = segno.make_qr('test')
    out = io.StringIO()
    scale = 1.34
    qr.save(out, kind='eps', scale=scale)
    assert '{0} {0} scale'.format(scale) in out.getvalue()


def eps_as_matrix(buff, border):
    """\
    Reads the path in the EPS and returns it as list of 0, 1 lists.

    :param io.StringIO buff: Buffer to read the matrix from.
    """
    eps = buff.getvalue()
    h, w = re.search(r'^%%BoundingBox: 0 0 ([0-9]+) ([0-9]+)', eps,
                     flags=re.MULTILINE).groups()
    if h != w:
        raise ValueError('Expected equal height/width, got height="{}" width="{}"'.format(h, w))
    size = int(w) - 2 * border
    path = re.search(r'^newpath[\s+](.+?)(^stroke)', eps,
                     flags=re.DOTALL | re.MULTILINE).group(1).strip()
    res = []
    res_row = None
    absolute_x = -border
    for x, y, op in re.findall(r'(-?[0-9]+(?:\.[0-9]+)?)\s+(-?[0-9]+(?:\.[0-9]+)?)\s+([a-z]+)', path):
        x = int(x)
        y = float(y)
        if y != 0.0:  # New row
            if res_row:
                res_row.extend([0] * (size - len(res_row)))
            res_row = []
            res.append(res_row)
        if op == 'moveto':
            absolute_x = 0
            if x != border:
                raise ValueError('Unexpected border width. Expected "{}", got "{}"'.format(border, x))
        else:
            absolute_x += x
            bit = 0 if op == 'm' else 1
            length = absolute_x if x < 0 else x
            res_row.extend([bit] * length)
    res_row.extend([0] * (size - len(res_row)))
    return res


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Shi Yan
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against Hanzi encoding.
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno
from segno import consts, encoder


def test_detect_hanzi():
    qr = segno.make('汉字')
    assert 'byte' == qr.mode
    assert qr.is_micro
    assert 'M3-M' == qr.designator


def test_default_hanzi_mode():
    qr = segno.make('书读百遍其义自现')
    assert 'byte' == qr.mode
    assert '2-M' == qr.designator


def test_force_hanzi_mode():
    qr = segno.make('书读百遍其义自现', mode='hanzi')
    assert 'hanzi' == qr.mode
    assert '1-M' == qr.designator


def test_default_hanzi_mode2():
    qr = segno.make('大江东去，浪淘尽，千古风流人物。故垒西边，人道是，三国周郎赤壁。乱石穿空，'
                    '惊涛拍岸，卷起千堆雪。江山如画，一时多少豪杰。遥想公瑾当年，小乔初嫁了，雄姿英发。'
                    '羽扇纶巾，谈笑间，樯橹灰飞烟灭。故国神游，多情应笑我，早生华发。人生如梦，一尊还酹江月。')
    assert 'byte' == qr.mode
    assert '12-L' == qr.designator


def test_force_hanzi_mode2():
    qr = segno.make('大江东去，浪淘尽，千古风流人物。故垒西边，人道是，三国周郎赤壁。乱石穿空，'
                    '惊涛拍岸，卷起千堆雪。江山如画，一时多少豪杰。遥想公瑾当年，小乔初嫁了，'
                    '雄姿英发。羽扇纶巾，谈笑间，樯橹灰飞烟灭。故国神游，多情应笑我，早生华发。人生如梦，一尊还酹江月。',
                    mode='hanzi')
    assert 'hanzi' == qr.mode
    assert '9-L' == qr.designator


def test_force_hanzi_mode_and_encoding():
    qr = segno.make('书读百遍其义自现', mode='hanzi', encoding='gb2312')
    assert 'hanzi' == qr.mode
    assert '1-M' == qr.designator


def test_default_utf8_encoder():
    qr = encoder.encode('书读百遍其义自现')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_BYTE == segment.mode
    assert 24 == segment.char_count


def test_force_hanzi_encoder():
    qr = encoder.encode('书读百遍其义自现', mode='hanzi')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_HANZI == segment.mode
    assert 8 == segment.char_count


def test_detect_hanzi_encoder2():
    # detect as utf8
    qr = encoder.encode('汉字')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_BYTE == segment.mode
    assert 6 == segment.char_count


def test_hanzi_bytes():
    qr = encoder.encode('书读百遍其义自现'.encode(consts.HANZI_ENCODING), mode='hanzi')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_HANZI == segment.mode
    assert 8 == segment.char_count


def test_not_hanzi():
    qr = segno.make_qr('Ä')
    assert 'byte' == qr.mode
    assert '1-H' == qr.designator


def test_not_hanzi2():
    qr = segno.make_qr('Ä'.encode('utf-8'))
    assert 'byte' == qr.mode
    assert '1-H' == qr.designator


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
LaTex output related tests.
"""
from __future__ import absolute_import, unicode_literals
import re
import io
import segno


def test_write_tex():
    # Test with default options
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='tex', border=4)
    assert r'\pgfpathmoveto{\pgfqpoint{4pt}{-4pt}}' in out.getvalue()


def test_write_tex_url():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    url = 'http://www.example.org/~xxx#aaa'
    qr.save(out, kind='tex', border=4, url=url)
    assert r'\href{' + url + '}' in out.getvalue()


def test_write_tex_omit_url():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    url = ''
    qr.save(out, kind='tex', border=4, url=url)
    assert r'\href' not in out.getvalue()


def test_write_tex_omit_url2():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='tex', border=4)
    assert r'\href' not in out.getvalue()


def test_write_tex_color():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='tex', border=4)
    assert r'\color' not in out.getvalue()


def test_write_tex_color2():
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='tex', border=4, dark='green')
    assert r'\color{green}' in out.getvalue()


_COMMAND_PATTERN = re.compile(r'pgfpath(move|line)to{\\pgfqpoint{(-?[0-9]+)pt}{(-?[0-9]+)pt}')


def tex_as_matrix(buff, border):
    """\
    Returns the LaTeX QR code as list of [0,1] lists.

    :param io.StringIO buff: Buffer to read the matrix from.
    """
    res = []
    last_y = None
    res_row = None
    size = 0
    prev_x = border
    for m in _COMMAND_PATTERN.finditer(buff.getvalue()):
        op, x, y = m.groups()
        x, y = int(x), int(y)
        length = x - prev_x
        prev_x = x
        if y != last_y:
            length = x - border
            if res_row is not None:
                res_row.extend([0] * (size - len(res_row)))
                if not size:
                    size = len(res_row)
            last_y = y
            res_row = []
            res.append(res_row)
        if op == 'move':
            res_row.extend([0] * length)
        elif op == 'line':
            res_row.extend([1] * length)
    res_row.extend([0] * (size - len(res_row)))
    return res


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
SVG related tests for multicolor support.
"""
from __future__ import absolute_import, unicode_literals
import io
import xml.etree.ElementTree as etree
import pytest
import segno
from segno import writers as colors

_SVG_NS = 'http://www.w3.org/2000/svg'


def _get_svg_el(root, name):
    return root.find('{%s}%s' % (_SVG_NS, name))


def _get_group(root):
    return _get_svg_el(root, 'g')


def _parse_xml(buff):
    """\
    Parses XML and returns the root element.
    """
    buff.seek(0)
    return etree.parse(buff).getroot()


def test_merge_colors():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='svg', dark='green', finder_dark='green',
            dark_module='green')
    green = colors._color_to_webcolor('green')
    assert green in out.getvalue().decode('utf-8')
    root = _parse_xml(out)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 1 == len(paths)


def test_merge_colors2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='svg', dark='green', finder_dark='green',
            dark_module='blue', alignment_light='yellow',
            quiet_zone='yellow')
    green = colors._color_to_webcolor('green')
    yellow = colors._color_to_webcolor('yellow')
    blue = colors._color_to_webcolor('blue')
    res = out.getvalue().decode('utf-8')
    assert green in res
    assert yellow in res
    assert blue in res
    root = _parse_xml(out)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 3 == len(paths)
    assert not any(p.attrib.get('transform') for p in paths)


def test_nogroup():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='svg', dark='green', finder_dark='green',
            dark_module='blue', alignment_light='yellow', quiet_zone='yellow',
            scale=1.0)
    root = _parse_xml(out)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 3 == len(paths)
    assert all(p.attrib.get('transform') is None for p in paths)
    group = _get_group(root)
    assert not group


def test_scale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='svg', dark='green', finder_dark='green',
            dark_module='blue', alignment_light='yellow', quiet_zone='yellow',
            scale=1.5)
    root = _parse_xml(out)
    paths = root.findall('.//{%s}path' % _SVG_NS)
    assert 3 == len(paths)
    assert all(p.attrib.get('transform') is None for p in paths)
    group = _get_group(root)
    assert group is not None
    assert 'scale(1.5)' == group.attrib.get('transform')


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #29.
<https://github.com/heuer/segno/issues/29>
"""
from __future__ import absolute_import, unicode_literals
import os
import io
import tempfile
import shutil
import pytest
import segno
from segno import encoder
try:
    from .tutils import read_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from tutils import read_matrix


def test_seq_behave_like_qrcode():
    qr = segno.make_qr('Something')
    sa = segno.QRCodeSequence([qr])
    assert 1 == len(sa)
    assert not qr.is_micro
    assert qr.is_micro == sa.is_micro
    assert qr.version == sa.version
    assert qr.error == sa.error
    assert qr.default_border_size == sa.default_border_size
    assert qr.mask == sa.mask
    assert qr.symbol_size() == sa.symbol_size()
    assert qr.is_micro == sa.is_micro
    assert qr == sa[0]
    assert qr.png_data_uri() == sa.png_data_uri()


def test_seq_dont_behave_like_qrcode():
    qr = segno.make_qr('Something')
    seq = segno.QRCodeSequence([qr, qr])
    assert 2 == len(seq)
    with pytest.raises(AttributeError):
        assert not seq.is_micro


_DATA_PARITY = (
    # Expected, Input
    # See ISO/IEC 18004:2015(E) page 61:
    # 30 ⊕ 31 ⊕ 32 ⊕ 33 ⊕ 34 ⊕ 35 ⊕ 36 ⊕ 37 ⊕ 38 ⊕ 39 ⊕ 93 ⊕ FA ⊕ 96 ⊕ 7B = 85
    (int('85', 16), '0123456789日本'),
    (int('85', 16), '01234日本56789'),
    (49, '123456789'),
    (49, 123456789),
    (160, 'Mürrisch'),
)


@pytest.mark.parametrize('expected, data', _DATA_PARITY)
def test_calc_sa_parity(expected, data):
    assert expected == encoder.calc_structured_append_parity(data)


def test_illegal_version():
    with pytest.raises(ValueError):
        segno.make_sequence('ABCD', version='M4')


def test_encode_single():
    # ISO/IEC 18004:2015(E) - page 7
    # 'QR Code Symbol' as 1-M symbol
    seq = segno.make_sequence('QR Code Symbol', version=1, error='M',
                              boost_error=False)
    assert '1-M' == seq.designator
    ref_matrix = read_matrix('iso-fig-1')
    assert ref_matrix == seq.matrix
    qr = seq[0]
    assert '1-M' == qr.designator
    ref_matrix = read_matrix('iso-fig-1')
    assert ref_matrix == qr.matrix


@pytest.mark.parametrize('version,symbol_count', [(None, 4), (1, None)])
def test_encode_multi_by_version_or_symbol_count(version, symbol_count):
    # ISO/IEC 18004:2015(E) - page 60
    seq = segno.make_sequence('ABCDEFGHIJKLMN'
                              'OPQRSTUVWXYZ0123'
                              '456789ABCDEFGHIJ'
                              'KLMNOPQRSTUVWXYZ',
                              version=version, symbol_count=symbol_count,
                              error='m', mask=4, boost_error=False)
    assert 4 == len(seq)
    ref_matrix = read_matrix('seq-iso-04-01')
    assert ref_matrix == seq[0].matrix
    ref_matrix = read_matrix('seq-iso-04-02')
    assert ref_matrix == seq[1].matrix
    ref_matrix = read_matrix('seq-iso-04-03')
    assert ref_matrix == seq[2].matrix
    ref_matrix = read_matrix('seq-iso-04-04')
    assert ref_matrix == seq[3].matrix


def test_too_much_for_one_qrcode():
    data = 'A' * 4296  # Version 40 supports max. 4296 alphanumeric chars (40-L)
    data += 'B'
    with pytest.raises(ValueError) as ex:
        segno.make(data)
    assert 'too large' in str(ex.value)


def test_dataoverflow():
    data = 'A' * 4296  # Version 40: max. 4296 alphanumeric chars
    seq = segno.make_sequence(data, version=40)
    assert 1 == len(seq)
    data += 'B'
    seq = segno.make_sequence(data, version=40)
    assert 2 == len(seq)


def test_dataoverflow_error():
    data = 'A' * 4296 * 16  # Version 40: max. 4296 alphanumeric chars * 16 symbols
    data = data[:-4]  # Remove some chars taking some SA overhead into account
    seq = segno.make_sequence(data, version=40)
    assert 16 == len(seq)
    data += 'B'
    with pytest.raises(segno.DataOverflowError) as ex:
        segno.make_sequence(data, version=40)
    assert 'does not fit' in str(ex.value)


def test_no_version_provided():
    with pytest.raises(ValueError):
        segno.make_sequence('A')


def test_int():
    data = int('1' * 42)
    seq = segno.make_sequence(data, version=1)
    assert 2 == len(seq)


def test_save_one():
    directory = tempfile.mkdtemp()
    assert 0 == len(os.listdir(directory))
    seq = segno.make_sequence('ABC', version=1)
    assert 1 == len(seq)
    seq.save(os.path.join(directory, 'test.svg'))
    number_of_files = len(os.listdir(directory))
    shutil.rmtree(directory)
    assert 1 == number_of_files


def test_save_multiple():
    directory = tempfile.mkdtemp()
    assert 0 == len(os.listdir(directory))
    seq = segno.make_sequence('ABCDEFGHIJKLMN'
                              'OPQRSTUVWXYZ0123'
                              '456789ABCDEFGHIJ'
                              'KLMNOPQRSTUVWXYZ', version=1, error='m')
    assert 4 == len(seq)
    seq.save(os.path.join(directory, 'test.svg'))
    number_of_files = len(os.listdir(directory))
    shutil.rmtree(directory)
    assert 4 == number_of_files


def test_save_terminal_one():
    out_multiple = io.StringIO()
    data = 'QR Code Symbol'
    seq = segno.make_sequence(data, version=1)
    assert 1 == len(seq)
    seq.terminal(out_multiple)
    qr = segno.make_qr(data, version=1)
    out_single = io.StringIO()
    qr.terminal(out_single)
    assert out_single.getvalue() == out_multiple.getvalue()


def test_save_terminal_multiple():
    out_multiple = io.StringIO()
    data = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    seq = segno.make_sequence(data, version=1, error='m')
    assert 4 == len(seq)
    seq.terminal(out_multiple)
    out_single = io.StringIO()
    for qr in seq:
        qr.terminal(out_single)
    assert out_single.getvalue() == out_multiple.getvalue()


def test_boosterror_noop():
    seq = segno.make_sequence('I read the news today oh boy', version=1)
    assert 2 == len(seq)
    assert 'L' == seq[0].error
    assert 'L' == seq[1].error


def test_boosterror():
    seq = segno.make_sequence('I read the news today oh boy / About a lucky man '
                              'who made the grade', version=2)
    assert 3 == len(seq)
    assert 'M' == seq[0].error
    assert 'M' == seq[1].error
    assert 'M' == seq[2].error


def test_boosterror2():
    seq = segno.make_sequence('I read the news today oh boy / About a lucky man '
                              'who made the grade', symbol_count=4)
    assert 4 == len(seq)
    assert '2-Q' == seq[0].designator
    assert '2-Q' == seq[1].designator
    assert '2-Q' == seq[2].designator
    assert '2-Q' == seq[3].designator


def test_toomany_symbols():
    with pytest.raises(ValueError):
        segno.make_sequence('ABCDEF', symbol_count=16)


@pytest.mark.parametrize('symbol_count', [0, -1, 17])
def test_illegal_symbolcount(symbol_count):
    with pytest.raises(ValueError):
        segno.make_sequence('I read the news today oh boy / About a lucky man '
                            'who made the grade',
                            symbol_count=symbol_count)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against Segno data URI.
"""
from __future__ import absolute_import
import segno


def test_data_svg():
    qr = segno.make_qr('A')
    val = qr.svg_data_uri()
    assert val
    expected = "data:image/svg+xml;charset=utf-8,%3Csvg%20xmlns%3D%27"
    assert expected == val[:len(expected)]
    assert val.endswith('%3C%2Fsvg%3E')


def test_data_svg_minimal_encoding():
    qr = segno.make_qr('A')
    val = qr.svg_data_uri(encode_minimal=True)
    assert val
    expected = "data:image/svg+xml;charset=utf-8,%3Csvg xmlns='"
    assert expected == val[:len(expected)]
    assert val.endswith('%3C/svg%3E')


def test_data_svg_no_charset():
    qr = segno.make_qr('A')
    val = qr.svg_data_uri(omit_charset=True)
    assert val
    expected = "data:image/svg+xml,%3Csvg%20xmlns%3D%27"
    assert expected == val[:len(expected)]
    assert val.endswith('%3C%2Fsvg%3E')


def test_data_png():
    qr = segno.make_qr('A')
    val = qr.png_data_uri()
    assert val
    assert val.startswith('data:image/png;base64,')


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against issue #45.
<https://github.com/heuer/segno/issues/45>
"""
from __future__ import unicode_literals, absolute_import
from segno import cli
import pytest


def test_issue_45_error(capsys):
    try:
        cli.main(['--version=M1', '--seq', '"This is a test"'])
    except SystemExit as ex:
        assert 1 == ex.code
        assert capsys.readouterr()


def test_issue_45_no_error():
    assert 0 == cli.main(['--version=1', '--seq',
                          '"This is a test, test test test test"'])


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PBM related tests.
"""
from __future__ import unicode_literals, absolute_import
import re
import io
from itertools import islice
import pytest
import segno


def test_p4():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pbm')
    assert out.getvalue().startswith(b'P4')


def test_not_plain():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pbm', plain=False)
    assert out.getvalue().startswith(b'P4')


def test_p1():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pbm', plain=True)
    assert out.getvalue().startswith(b'P1')


_is_size = re.compile(br'^([0-9]+)\s+[0-9]+$').match


def _move_to_raster(buff, border):
    """\
    Returns an iterator and the si
    """
    code = buff.getvalue().splitlines()
    len_without_border = 0
    code_iter = iter(code)
    for line in code_iter:
        if line.startswith(b'P') or line.startswith(b'#'):
            continue
        m = _is_size(line)
        if m:
            len_without_border = int(m.group(1)) - border
            break
    return code_iter, len_without_border


def pbm_p1_as_matrix(buff, border):
    """\
    Returns the text QR code as list of [0, 1] lists.

    :param io.BytesIO buff: Buffer to read the matrix from.
    :param int border: The QR code border
    """
    res = []
    code_iter, len_without_border = _move_to_raster(buff, border)
    for line in islice(code_iter, border, len_without_border):
        row = [int(i) for i in islice(line.decode('ascii'), border,
                                      len_without_border)]
        res.append(row)
    return res


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests plugin loading.
"""
from __future__ import absolute_import, unicode_literals
import os
import pkg_resources
import pytest
import segno


def test_noplugin():
    qr = segno.make('The Beatles')
    with pytest.raises(AttributeError):
        qr.to_unknown_plugin()


def an_example_plugin(qrcode):
    """\
    This is a Segno converter plugin used by the next test case.
    """
    assert qrcode
    return 'works'


def test_plugin():
    distribution = pkg_resources.Distribution(os.path.dirname(__file__))
    entry_point = pkg_resources.EntryPoint.parse('test = test_plugin:an_example_plugin', dist=distribution)
    distribution._ep_map = {'segno.plugin.converter': {'test': entry_point}}
    pkg_resources.working_set.add(distribution)

    qr = segno.make('The Beatles')
    assert 'works' == qr.to_test()


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the ``utils`` module.
"""
from __future__ import absolute_import, unicode_literals
import pytest
from segno import utils, consts


def test_get_border():
    border = utils.get_border(1, None)
    assert 4 == border
    border = utils.get_border(1, 3)
    assert 3 == border


def test_get_border2():
    border = utils.get_border(consts.VERSION_M1, 1)
    assert 1 == border
    border = utils.get_border(consts.VERSION_M1, None)
    assert 2 == border


def test_get_border3():
    border = utils.get_border(3, 0)
    assert 0 == border
    border = utils.get_border(3, None)
    assert 4 == border


def test_get_symbol_size():
    version = 1
    matrix_size = 21
    border = 0
    width, height = utils.get_symbol_size(version, border=border)
    assert (matrix_size, matrix_size) == (width, height)
    border = 4
    width, height = utils.get_symbol_size(1)
    assert (matrix_size + 2 * border, matrix_size + 2 * border) == (width, height)


def test_get_symbol_size_micro():
    version = consts.VERSION_M2
    matrix_size = 13
    border = 0
    width, height = utils.get_symbol_size(version, border=border)
    assert (matrix_size, matrix_size) == (width, height)
    border = 2
    width, height = utils.get_symbol_size(version)
    assert (matrix_size + 2 * border, matrix_size + 2 * border) == (width, height)


@pytest.mark.parametrize('scale', [1, 1.2, .8, 10])
def test_valid_scale(scale):
    assert utils.check_valid_scale(scale) is None


@pytest.mark.parametrize('scale', (0.0, 0, -1, -.2, int(.8)))
def test_invalid_scale(scale):
    with pytest.raises(ValueError):
        utils.check_valid_scale(scale)


@pytest.mark.parametrize('border', (None, 0, 0.0, 1, 2))
def test_valid_border(border):
    assert utils.check_valid_border(border) is None


@pytest.mark.parametrize('border', (.2, -1, 1.3))
def test_invalid_border(border):
    with pytest.raises(ValueError):
        utils.check_valid_border(border)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the encoder module.
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno
from segno import consts

_DATA_AUTODETECT = (
    # Input, expected version, expected mode
    ('123456', 'numeric'),
    (123456, 'numeric'),
    (+123456, 'numeric'),
    ('+123456', 'alphanumeric'),
    (-1234, 'alphanumeric'),
    ('-1234', 'alphanumeric'),
    ('123A', 'alphanumeric'),
    ('123a', 'byte'),
    (consts.ALPHANUMERIC_CHARS, 'alphanumeric'),
    ('HELLO WORLD', 'alphanumeric'),
    ('HELLO\nWORLD', 'byte'),
    ('MÄRCHENBUCH', 'byte'),
    ('ABCDEFGHIJ1234567890\n', 'byte'),
    ('®', 'byte'),
    ('http://www.example.org/', 'byte'),
    ('http://www.example.org/path/index.html', 'byte'),
    ('点', 'kanji'),
    ('茗', 'kanji'),
    ('漢字', 'kanji'),
    ('外来語', 'kanji'),
    ('外来語'.encode('utf-8'), 'byte'),
)


@pytest.mark.parametrize('data, expected_mode', _DATA_AUTODETECT)
def test_valid_mode_autodetection(data, expected_mode):
    qr = segno.make_qr(data)
    assert expected_mode == qr.mode


@pytest.mark.parametrize('data, expected_mode', _DATA_AUTODETECT)
def test_valid_mode_autodetection_auto(data, expected_mode):
    qr = segno.make(data)
    assert expected_mode == qr.mode


def test_default_encoding():
    qr = segno.make('Märchenbücher', error='m', micro=False)
    # 1 since the data fits into version 1 if ISO/IEC 8859-1 (the default
    # encoding) is used
    assert 1 == qr.version
    assert 'byte' == qr.mode


def test_encoding_latin1():
    qr = segno.make('Märchenbücher', error='m', encoding='latin1', micro=False)
    assert 1 == qr.version
    assert 'byte' == qr.mode


def test_encoding_utf8():
    qr = segno.make('Märchenbücher', error='m', encoding='utf-8', micro=False)
    assert 2 == qr.version
    assert 'byte' == qr.mode


def test_kanji_enforce_byte():
    data = '点'
    qr = segno.make_qr(data)
    assert 'kanji' == qr.mode
    qr = segno.make_qr(data, encoding='utf-8')
    assert 'byte' == qr.mode


def test_kanji_enforce_byte2():
    data = '点'
    qr = segno.make_qr(data.encode('utf-8'))
    assert 'byte' == qr.mode


def test_kanji_bytes():
    data = '外来語'
    qr = segno.make_qr(data.encode('shift_jis'))
    assert 'kanji' == qr.mode


def test_kanji_mode_byte():
    data = '外来語'
    qr = segno.make_qr(data, mode='byte')
    assert 'byte' == qr.mode


def test_kanji_mode_byte2():
    data = '漢字'.encode('shift_jis')
    qr = segno.make_qr(data, mode='byte')
    assert 'byte' == qr.mode


def test_create_micro():
    qr = segno.make_micro('1')
    assert qr.is_micro
    assert 'M1' == qr.version


def test_make_micro_overflow():
    data = 'A' * 21  # Max. 21 alphanumeric chars (M4-L)
    qr = segno.make(data, micro=True)
    assert 'M4-L' == qr.designator
    data += 'A'
    with pytest.raises(segno.DataOverflowError) as ex:
        segno.make(data, micro=True)
    assert 'too large' in str(ex.value)


def test_enforce_qrcode():
    content = 'HELLO WORLD'
    qr = segno.make(content)
    assert qr.is_micro
    assert 'M3' == qr.version
    qr = segno.make_qr(content)
    assert not qr.is_micro
    assert 1 == qr.version
    qr = segno.make(content, micro=False)
    assert not qr.is_micro
    assert 1 == qr.version


def test_m1_has_no_error_level():
    qr = segno.make('1')
    assert qr.is_micro
    assert 'M1' == qr.version
    assert qr.error is None


def test_micro_invalid_error():
    with pytest.raises(ValueError) as ex:
        segno.make_micro(1, error='h')
    assert 'not available' in str(ex.value)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Different output tests.
"""
from __future__ import unicode_literals, absolute_import
import io
import pytest
import segno
try:
    from .test_eps import eps_as_matrix
    from .test_png import png_as_matrix
    from .test_svg import svg_as_matrix
    from .test_txt import txt_as_matrix
    from .test_pdf import pdf_as_matrix
    from .test_terminal import terminal_as_matrix
    from .test_pbm import pbm_p1_as_matrix
    from .test_pam import pam_bw_as_matrix
    from .test_ppm import ppm_bw_as_matrix
    from .test_tex import tex_as_matrix
    from .test_xpm import xpm_as_matrix
# Attempted relative import in non-package
except (ValueError, SystemError, ImportError):
    from test_eps import eps_as_matrix
    from test_png import png_as_matrix
    from test_svg import svg_as_matrix
    from test_txt import txt_as_matrix
    from test_pdf import pdf_as_matrix
    from test_terminal import terminal_as_matrix
    from test_pbm import pbm_p1_as_matrix
    from test_pam import pam_bw_as_matrix
    from test_ppm import ppm_bw_as_matrix
    from test_tex import tex_as_matrix
    from test_xpm import xpm_as_matrix


_DATA = (
    # Input string, error level, border
    ('Märchenbuch', 'M', 4),
    (123, 'H', 0),
    ('http:/www.example.org/', 'L', 3),
    ('Hello\nWorld', 'Q', 2),
    ('HELLO WORLD', 'H', 2),
    ('外来語',       'L', 0),
)


def _make_test_data_input():
    for kind, buffer_factory, to_matrix_func, kw in (
        ('eps', io.StringIO, eps_as_matrix, {}),
        ('png', io.BytesIO, png_as_matrix, {}),
        ('svg', io.BytesIO, svg_as_matrix, {}),
        ('txt', io.StringIO, txt_as_matrix, {}),
        ('pdf', io.BytesIO, pdf_as_matrix, {}),
        ('ans', io.StringIO, terminal_as_matrix, {}),
        ('tex', io.StringIO, tex_as_matrix, {}),
        ('xpm', io.StringIO, xpm_as_matrix, {}),
        ('pam', io.BytesIO, pam_bw_as_matrix, {}),
        ('pbm', io.BytesIO, pbm_p1_as_matrix, dict(plain=True)),
        ('ppm', io.BytesIO, ppm_bw_as_matrix, {}),
    ):
        for data, error, border in _DATA:
            yield kind, buffer_factory, to_matrix_func, data, error, border, kw


@pytest.mark.parametrize('kind, buffer_factory, to_matrix_func, data, error, '
                         'border, kw',
                         _make_test_data_input())
def test_data(kind, buffer_factory, to_matrix_func, data, error, border, kw):
    # Creates a QR Code, serializes it and checks if the serialization
    # corresponds to the initial QR Code matrix.
    """\
    :param str kind: "kind" parameter to serialize the QR code
    :param buffer_factory: Callable to construct the output buffer.
    :param to_matrix_func: Function to convert the buffer back to a matrix.
    :param data: The input to construct the QR code.
    :param error: ECC level
    :param int border: Border size.
    """
    qr = segno.make_qr(data, error=error)
    out = buffer_factory()
    qr.save(out, kind=kind, border=border, **kw)
    matrix = to_matrix_func(out, border)
    assert len(qr.matrix) == len(matrix)
    for i, row in enumerate(qr.matrix):
        exptected_row = bytearray(matrix[i])
        assert len(row) == len(exptected_row)
        assert exptected_row == row, 'Error in row {0}'.format(i)


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Terminal output related tests.
"""
from __future__ import absolute_import, unicode_literals, division
import io
import re
import pytest
import segno


def test_terminal():
    # Test with default options
    qr = segno.make_qr('test')
    # Upper left finder pattern: 7 dark modules + 1 light module
    expected = '\033[49m' + '  ' * 7 + '\033[0m\033[7m '
    out = io.StringIO()
    qr.terminal(out, border=0)
    val = out.getvalue()
    assert expected == val[:len(expected)]


def test_terminal_compact():
    # Test compact half-block terminal QR.
    qr = segno.make_qr('test')
    out = io.StringIO()
    qr.terminal(out, border=0, compact=True)
    val = out.getvalue()
    expected = """\
 ▄▄▄▄▄ █ ▀ ▀▀█ ▄▄▄▄▄ 
 █   █ █▄██  █ █   █ 
 █▄▄▄█ █▀ ▀███ █▄▄▄█ 
▄▄▄▄▄▄▄█ ▀ █ █▄▄▄▄▄▄▄
▀▀▄  ▀▄▀ ▀█▄▄ ▄▄█▀ ▄ 
█ ▄▄ █▄ ▀▀ ▀▄█ ▄█  ▄█
▄▄██▄█▄█▀▀ ▄██▀ █▄▀ ▀
 ▄▄▄▄▄ ██ ▀▄ █▀▀  ▀▄█
 █   █ █ ▀   ▀▀▀▄ ▀▄█
 █▄▄▄█ █▄█▄▄ █▄ ██ ██
       ▀▀ ▀▀  ▀ ▀▀  ▀
"""  # noqa: W291
    assert expected == val


def terminal_as_matrix(buff, border):
    """\
    Returns the text QR code as list of [0,1] lists.
    """
    color_pattern = re.compile(r'(\033\[\d+m)(\s+)\033\[0m')
    res = []
    colors = ('\033[7m', '\033[49m')
    code = buff.getvalue().splitlines()
    for line in code[border:len(code) - border]:
        row = []
        for m in color_pattern.finditer(line):
            bit = colors.index(m.group(1))
            bit_count = len(m.group(2)) // 2  # 2 chars for 1 module!
            if m.start() == 0 or m.end() == len(line):
                bit_count -= border
            row.extend([bit] * bit_count)
        res.append(row)
    return res


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Text output related tests.
"""
from __future__ import absolute_import, unicode_literals
import io
from itertools import islice
import segno


def test_write_txt():
    # Test with default options
    qr = segno.make_qr('test', error='m', boost_error=False)
    out = io.StringIO()
    qr.save(out, kind='txt', border=0)
    expected = '11111110011'
    val = out.getvalue()
    assert expected == val[:len(expected)]


def txt_as_matrix(buff, border):
    """\
    Returns the text QR code as list of [0,1] lists.

    :param io.StringIO buff: Buffer to read the matrix from.
    """
    res = []
    code = buff.getvalue().splitlines()
    len_without_border = len(code) - border
    for line in islice(code, border, len_without_border):
        res.append([int(clr) for clr in islice(line, border,
                                               len_without_border)])
    return res


if __name__ == '__main__':
    import pytest
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PPM related tests.
"""
from __future__ import unicode_literals, absolute_import
import re
import io
from struct import unpack
import pytest
import segno
try:
    range = xrange  # noqa
except NameError:
    pass


def test_invalid_color():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='ppm', dark=None)


def test_not_plain():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='ppm')
    assert out.getvalue().startswith(b'P6')


_size = re.compile(br'^P6\s+(?:#[^\n]+\s*)([0-9]+)\s+(?:[0-9]+\s+[0-9]+\n)').match


def _image_data(buff):
    """\
    Returns the image data and the size of the matrix.
    """
    code = buff.getvalue()
    m = _size(code)
    if m:
        size = int(m.group(1))
    else:
        raise Exception('Internal error: PPM header not found')
    return code[m.end():], size


def ppm_bw_as_matrix(buff, border):
    """\
    Returns the QR code as list of [0, 1] lists.

    :param io.BytesIO buff: Buffer to read the matrix from.
    """
    res = []
    data, size = _image_data(buff)
    rgb_data = [unpack('>3B', data[i:i + 3]) for i in range(0, len(data), 3)]
    for i, offset in enumerate(range(0, len(rgb_data), size)):
        if i < border:
            continue
        if i >= size - border:
            break
        row_data = rgb_data[offset + border:offset + size - border]
        res.append([(0x0, 0x1)[rgb == (0, 0, 0)] for rgb in row_data])
    return res


if __name__ == '__main__':
    pytest.main([__file__])



from django.db import models


class Ticket(models.Model):
    """Example which stores an alphanumeric name and the name as QR code.
    """
    name = models.CharField(max_length=150, unique=True)
    qrcode = models.ImageField(upload_to='ticket-qrcodes/')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the colors module.
"""
from __future__ import absolute_import, unicode_literals
import pytest
from segno import writers as colors


def test_illegal():
    with pytest.raises(ValueError):
        colors._color_to_rgb('unknown')


def test_illegal2():
    with pytest.raises(ValueError):
        colors._color_to_rgb((1, 2, 3, 256))


def test_illegal3():
    with pytest.raises(ValueError):
        colors._color_to_rgb((300, 300, 300))


def test_illegal4():
    with pytest.raises(ValueError):
        colors._color_to_rgb((0, 0, 256))


def test_illegal5():
    with pytest.raises(ValueError):
        colors._color_to_rgb((256, 0, 0))


def test_rgba_vs_rgb_conflict():
    with pytest.raises(ValueError):
        colors._color_to_rgb('#949494E8')


@pytest.mark.parametrize('clr, expected', [((0, 0, 0, 0), False),
                                           ((0, 0, 0, 1), True),
                                           ((0, 0, 0, 1.0), True),
                                           ((0, 0, 0, 255), True),
                                           ((0, 0, 0, 0.25), False),
                                           ('#000', True),
                                           ('#000000', True),
                                           ('Black', True),
                                           ('black', True),
                                           ('BLACK', True),
                                           ('blacK', True),
                                           ])
def test_color_is_black(clr, expected):
    assert expected == colors._color_is_black(clr)


@pytest.mark.parametrize('clr, expected', (((255, 255, 255, 0), False),
                                           ((255, 255, 255, 1), True),
                                           ((255, 255, 255, 255), True),
                                           ((255, 255, 255, 1.0), True),
                                           ((255, 255, 255, .0), False),
                                           ((255, 255, 255, .25), False),
                                           ('#FFF', True),
                                           ('#fFF', True),
                                           ('#ffF', True),
                                           ('#fff', True),
                                           ('#ffffff', True),
                                           ('White', True),
                                           ('white', True),
                                           ('WHITE', True),
                                           ('whitE', True),
                                           ))
def test_color_is_white(clr, expected):
    assert expected == colors._color_is_white(clr)


@pytest.mark.parametrize('clr, expected', (('black', '#000'),
                                           ('WHite', '#fff'),
                                           ('#000000', '#000'),
                                           ('#ffFFff', '#fff'),
                                           ('#EEeeEE', '#eee'),
                                           ('#F00', 'red'),
                                           ('#FF0000', 'red'),
                                           ('red', 'red'),
                                           ('#d2b48c', 'tan'),
                                           ('tan', 'tan'),
                                           ((0, 0, 0, 1.0), '#000'),
                                           ((255, 255, 255, 1.0), '#fff'),
                                           ((255, 0, 0, 0.25), 'rgba(255,0,0,0.25)'),
                                           ('#0000ffcc', 'rgba(0,0,255,0.8)'),
                                           ('#949494E8', 'rgba(148,148,148,0.91)'),
                                           ))
def test_color_to_webcolor(clr, expected):
    assert expected == colors._color_to_webcolor(clr)


@pytest.mark.parametrize('clr, expected', (('black', '#000'),
                                           ('#F00', '#ff0000'),
                                           ('#FF0000', '#ff0000'),
                                           ('red', '#ff0000'),
                                           ('#D2B48C', '#d2b48c'),
                                           ((0, 0, 0, 1.0), '#000'),
                                           ((255, 255, 255, 1.0), '#fff'),
                                           ))
def test_color_to_webcolor_dont_optimize(clr, expected):
    assert expected == colors._color_to_webcolor(clr, optimize=False)


def _make_valid_colornames_data():
    data = (
        ('red', (255, 0, 0)),
        ('green', (0, 128, 0)),
        ('blue', (0, 0, 255)),
        ('Fuchsia', (255, 0, 255)),
        ('CoRnFloWeRblUe', (100, 149, 237)),
        ('hOtPink', (255, 105, 180)),
        ('darkSlateGrey', (47, 79, 79)),
    )
    for name, expected in data:
        yield name, expected
        yield name.title(), expected
        yield name.upper(), expected
        yield name.lower(), expected


@pytest.mark.parametrize('name, expected', _make_valid_colornames_data())
def test_valid_colornames(name, expected):
    rgb = colors._color_to_rgb(name)
    assert 3 == len(rgb)
    assert expected == rgb


@pytest.mark.parametrize('color, expected', (('#fff', (255, 255, 255)),
                                             ('#0000ffcc', (0, 0, 255, .8)),
                                             ('#949494E8', (148, 148, 148, 0.91)),
                                             ))
def test_hex_to_rgba(color, expected):
    assert expected == colors._hex_to_rgb_or_rgba(color)


@pytest.mark.parametrize('color, expected', (('#fff', (255, 255, 255)),
                                             ('#0000ffcc', (0, 0, 255, 204)),
                                             ('#949494E8', (148, 148, 148, 232)),
                                             ))
def test_hex_to_rgba_alpha_int(color, expected):
    assert expected == colors._hex_to_rgb_or_rgba(color, alpha_float=False)


def _make_valid_hexcodes_rgb_data():
    data = (
        ('#000', (0, 0, 0)),
        ('#FF1493', (255, 20, 147)),
        ('#FA8072', (250, 128, 114)),
        ('00F', (0, 0, 255)),
        ('#800000', (128, 0, 0)),
        ('#812dd3', (129, 45, 211)),
    )
    for name, expected in data:
        yield name, expected
        yield name.title(), expected
        yield name.upper(), expected
        yield name.lower(), expected


@pytest.mark.parametrize('name, expected', _make_valid_hexcodes_rgb_data())
def test_valid_hexcodes_rgb(name, expected):
    rgb = colors._color_to_rgb(name)
    assert 3 == len(rgb)
    assert expected == rgb


def _make_valid_hexcodes_rgba_data():
    data = (
        ('#808000', (128, 128, 0, 1.0)),
        ('red', (255, 0, 0, 1.0)),
    )
    for name, expected in data:
        yield name, expected
        yield name.title(), expected
        yield name.upper(), expected
        yield name.lower(), expected


@pytest.mark.parametrize('name, expected', _make_valid_hexcodes_rgba_data())
def test_valid_hexcodes_rgba(name, expected):
    rgba = colors._color_to_rgba(name)
    assert 4 == len(rgba)
    assert expected == rgba


@pytest.mark.parametrize('t, expected', (
                                            ('#808000', (128, 128, 0, 1.0)),
                                            ('red', (255, 0, 0, 1.0)),
                                            ((255, 0, 0, .2), (255, 0, 0, .2)),
    ))
def test_tuple_to_rgba(t, expected):
    rgba = colors._color_to_rgba(t)
    assert expected == rgba


@pytest.mark.parametrize('t, expected', (
                                        ('#808000', (128, 128, 0, 255)),
                                        ('red', (255, 0, 0, 255)),
                                        ((0, 0, 255, .8), (0, 0, 255, 204)),
    ))
def test_tuple_to_rgba_int(t, expected):
    rgba = colors._color_to_rgba(t, alpha_float=False)
    assert expected == rgba


@pytest.mark.parametrize('color, expected', (
                                            ((0, 0, 0), (255, 255, 255)),
                                            ((255, 255, 255), (0, 0, 0)),
                                            ((123, 123, 123), (132, 132, 132)),
                                            ((60, 70, 80), (195, 185, 175)),
    ))
def test_invert_color(color, expected):
    assert expected == colors._invert_color(color)


if __name__ == '__main__':
    pytest.main([__file__])



import io
from django.http import HttpResponse, HttpResponseRedirect
from django.core.files.base import ContentFile
from django.shortcuts import render
import segno
from .models import Ticket
from .forms import TicketForm


def index(request):
    """Renders the form to create a ticket"""
    if request.method == 'POST':
        form = TicketForm(request.POST)
        if form.is_valid():
            name = form.cleaned_data['name']
            qr = segno.make_qr(name)
            buff = io.BytesIO()
            qr.save(buff, kind='png', scale=3, dark='darkblue')
            ticket = Ticket(name=name)
            ticket.qrcode.save(name + '.png', ContentFile(buff.getvalue()),
                               save=True)
            return HttpResponseRedirect('/thanks/')
    else:
        form = TicketForm()
    return render(request, 'qrcode/example.html', {'form': form})


def thanks(request):
    return HttpResponse('Thanks, a new ticket was created')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
CLI colormap (PNG) related tests.
"""
from __future__ import unicode_literals, absolute_import
import io
import os
import tempfile
import pytest
from segno import cli, writers as colors
from png import Reader as PNGReader


def _make_tmp_png_filename():
    f = tempfile.NamedTemporaryFile('w', suffix='.png', delete=False)
    f.close()
    return f.name


def test_greyscale():
    fn = _make_tmp_png_filename()
    res = cli.main(['--quiet-zone=white', '--output={0}'.format(fn), 'test'])
    with open(fn, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fn)
    assert 0 == res
    reader = PNGReader(file=data)
    reader.preamble()
    assert reader.greyscale


def test_not_greyscale():
    fn = _make_tmp_png_filename()
    res = cli.main(['--quiet-zone=transparent', '--output={0}'.format(fn),
                    'test'])
    with open(fn, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fn)
    assert 0 == res
    reader = PNGReader(file=data)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 3 == len(palette)
    assert 0 == palette[0][3]  # Transparent color
    assert (0, 0, 0, 255) in palette  # black
    assert (255, 255, 255, 255) in palette  # white


def test_plte_colors():
    fn = _make_tmp_png_filename()
    res = cli.main(['--quiet-zone=green', '--finder-dark=purple',
                    '--finder-light=yellow', '--output={0}'.format(fn), 'test'])
    with open(fn, 'rb') as f:
        data = io.BytesIO(f.read())
    os.unlink(fn)
    assert 0 == res
    reader = PNGReader(file=data)
    reader.preamble()
    assert not reader.greyscale
    palette = reader.palette()
    assert 5 == len(palette)
    assert (0, 0, 0) in palette
    assert (255, 255, 255) in palette
    assert colors._color_to_rgb('green') in palette
    assert colors._color_to_rgb('purple') in palette
    assert colors._color_to_rgb('yellow') in palette


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Test against issue #32.
<https://github.com/heuer/segno/issues/32>

SVG version 2.0 shouldn't have a version attribute.
"""
from __future__ import absolute_import, unicode_literals
import io
import xml.etree.ElementTree as etree
import pytest
import segno

_SVG_NS = 'http://www.w3.org/2000/svg'


def _parse_xml(buff):
    """\
    Parses XML and returns the root element.
    """
    buff.seek(0)
    return etree.parse(buff).getroot()


def _get_svg_el(root, name):
    return root.find('{%s}%s' % (_SVG_NS, name))


def _get_path(root):
    return _get_svg_el(root, 'path')


def test_version_none():
    qr = segno.make('Test')
    out = io.BytesIO()
    qr.save(out, kind='svg', dark='#0000ffcc')
    root = _parse_xml(out)
    assert root.get('version') is None
    assert b'stroke-opacity' in out.getvalue()
    assert b'rgba(' not in out.getvalue()


def test_version_1_1():
    qr = segno.make('Test')
    out = io.BytesIO()
    qr.save(out, svgversion=1.1, kind='svg', dark='#0000ffcc')
    root = _parse_xml(out)
    assert '1.1' == root.get('version')
    assert b'stroke-opacity' in out.getvalue()
    assert b'rgba(' not in out.getvalue()


def test_version_2_0():
    qr = segno.make('Test')
    out = io.BytesIO()
    qr.save(out, svgversion=2.0, kind='svg', dark='#0000ffcc')
    root = _parse_xml(out)
    assert root.get('version') is None
    assert b'stroke-opacity' not in out.getvalue()
    assert b'rgba(' in out.getvalue()


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
PDF related tests.
"""
from __future__ import absolute_import, unicode_literals
import re
import io
import zlib
import pytest
import segno


def test_default_scale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    scale = 1
    scale_cmd = '{0} 0 0 {0} 0 0 cm'.format(scale)
    qr.save(out, kind='pdf', compresslevel=0)
    assert scale_cmd not in _find_graphic(out)


def test_scale():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    scale = 2
    scale_cmd = '{0} 0 0 {0} 0 0 cm'.format(scale)
    qr.save(out, kind='pdf', scale=scale, compresslevel=0)
    assert scale_cmd in _find_graphic(out)


def test_scale_float():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    scale = 1.34
    scale_cmd = '{0} 0 0 {0} 0 0 cm'.format(scale)
    qr.save(out, kind='pdf', scale=scale, compresslevel=0)
    assert scale_cmd in _find_graphic(out)


def test_background_none():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf')
    graphic = _find_graphic(out)
    assert 'rg' not in graphic
    assert 're' not in graphic


def test_background_set():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf', light='yellow')
    graphic = _find_graphic(out)
    assert 'rg' in graphic
    assert 're' in graphic


def test_stokecolor_default():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf')
    graphic = _find_graphic(out)
    assert 'RG' not in graphic


def test_stokecolor_black():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf', dark='black')
    graphic = _find_graphic(out)
    assert 'RG' not in graphic


def test_stokecolor_black2():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf', dark='#000')
    graphic = _find_graphic(out)
    assert 'RG' not in graphic


def test_stokecolor_set():
    qr = segno.make_qr('test')
    out = io.BytesIO()
    qr.save(out, kind='pdf', dark='#EEE')
    graphic = _find_graphic(out)
    assert 'RG' in graphic


def test_illegal_color_float():
    color = (.1, 1.1, .1)
    qr = segno.make_qr('test')
    out = io.BytesIO()
    with pytest.raises(ValueError):
        qr.save(out, kind='pdf', dark=color)


def _find_graphic(out):
    val = out.getvalue()
    start = b'stream\r\n'
    compressed_graphic = val[val.find(start) + len(start):val.find(b'\r\nendstream')]
    return zlib.decompress(compressed_graphic).decode('ascii')


def pdf_as_matrix(buff, border):
    """\
    Reads the path in the PDF and returns it as list of 0, 1 lists.

    :param io.BytesIO buff: Buffer to read the matrix from.
    :param int border: The QR code border
    """
    pdf = buff.getvalue()
    h, w = re.search(br'/MediaBox \[0 0 ([0-9]+) ([0-9]+)]', pdf,
                     flags=re.MULTILINE).groups()
    if h != w:
        raise ValueError('Expected equal height/width, got height="{}" width="{}"'.format(h, w))
    size = int(w) - 2 * border

    graphic = _find_graphic(buff)
    res = [[0] * size for i in range(size)]
    for x1, y1, x2, y2 in re.findall(r'\s*(-?\d+)\s+(-?\d+)\s+m\s+'
                                     r'(-?\d+)\s+(-?\d+)\s+l', graphic):
        x1, y1, x2, y2 = [int(i) for i in (x1, y1, x2, y2)]
        y = abs(y1)
        res[y][x1:x2] = [1] * (x2 - x1)
    return res


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against Kanji encoding.
"""
from __future__ import absolute_import, unicode_literals
import pytest
import segno
from segno import consts, encoder


def test_detect_kanji():
    qr = segno.make('続きを読む')
    assert 'kanji' == qr.mode
    assert qr.is_micro
    assert 'M3-L' == qr.designator


def test_detect_kanji_encoder():
    qr = encoder.encode('続きを読む')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_KANJI == segment.mode
    assert 5 == segment.char_count


def test_detect_kanji_encoder2():
    qr = encoder.encode('漢字')
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_KANJI == segment.mode
    assert 2 == segment.char_count


def test_kanji_bytes():
    qr = encoder.encode('続きを読む'.encode(consts.KANJI_ENCODING))
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_KANJI == segment.mode
    assert 5 == segment.char_count


def test_encode_kanji_byte():
    qr = encoder.encode('漢字'.encode(consts.KANJI_ENCODING))
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_KANJI == segment.mode
    assert 2 == segment.char_count


def test_encode_kanji_byte2():
    qr = encoder.encode('漢字'.encode(consts.KANJI_ENCODING),
                        encoding=consts.KANJI_ENCODING)
    assert 1 == len(qr.segments)
    segment = qr.segments[0]
    assert consts.MODE_KANJI == segment.mode
    assert 2 == segment.char_count


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Executes all tests.
"""
if __name__ == '__main__':
    import pytest
    pytest.main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
SVG embeddable into HTML tests.
"""
from __future__ import absolute_import, unicode_literals
import xml.etree.ElementTree as etree
import pytest
import segno
_PY2 = False
try:
    unicode
    _PY2 = True
except NameError:
    pass

_CSS_CLASS = 'segno'
_PATH_CLASS = 'qrline'


def _get_svg_el(root, name):
    return root.find('%s' % name)


def _get_group(root):
    return _get_svg_el(root, 'g')


def _get_first_path(root):
    g = _get_group(root)
    return _get_svg_el(root if g is None else g, 'path')


def _get_title(root):
    return _get_svg_el(root, 'title')


def _get_desc(root):
    return _get_svg_el(root, 'desc')


def _parse_xml(s):
    """\
    Parses XML and returns the root element.
    """
    if _PY2:
        s = s.encode('utf-8')
    return etree.fromstring(s)


def test_write_svg():
    qr = segno.make_qr('test')
    svg = qr.svg_inline()
    assert svg.startswith('<svg')
    root = _parse_xml(svg)
    # No background (and scaling) -> no group
    assert _get_group(root) is None
    assert 'viewBox' not in root.attrib
    assert 'height' in root.attrib
    assert 'width' in root.attrib
    css_class = root.attrib.get('class')
    assert css_class
    assert _CSS_CLASS == css_class
    path_el = _get_first_path(root)
    assert path_el is not None
    path_class = path_el.get('class')
    assert _PATH_CLASS == path_class
    stroke = path_el.get('stroke')
    assert stroke == '#000'
    title_el = _get_title(root)
    assert title_el is None
    desc_el = _get_desc(root)
    assert desc_el is None


@pytest.mark.parametrize('dark', ['bLack', '#000000', (0, 0, 0)])
def test_write_svg_black(dark):
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark=dark)
    assert svg.startswith('<svg')
    root = _parse_xml(svg)
    # No background (and scaling) -> no group
    assert _get_group(root) is None
    assert 'viewBox' not in root.attrib
    assert 'height' in root.attrib
    assert 'width' in root.attrib
    css_class = root.attrib.get('class')
    assert css_class
    assert _CSS_CLASS == css_class
    path_el = _get_first_path(root)
    assert path_el is not None
    path_class = path_el.get('class')
    assert _PATH_CLASS == path_class
    stroke = path_el.get('stroke')
    assert stroke == '#000'
    title_el = _get_title(root)
    assert title_el is None
    desc_el = _get_desc(root)
    assert desc_el is None


def test_write_svg_background_omitted():
    qr = segno.make_qr('test')
    svg = qr.svg_inline()
    assert svg.startswith('<svg')
    root = _parse_xml(svg)
    # No background (and scaling) -> no group
    assert _get_group(root) is None
    # Background should be the first path in the doc
    path = _get_first_path(root)
    assert path is not None
    assert not path.attrib.get('fill')


@pytest.mark.parametrize('light', ['wHitE', '#fff', (255, 255, 255), '#ffffff'])
def test_write_svg_background_white(light):
    qr = segno.make_qr('test')
    svg = qr.svg_inline(light=light)
    assert svg.startswith('<svg')
    root = _parse_xml(svg)
    # No scaling -> no group
    assert _get_group(root) is None
    # Background should be the first path in the doc
    path = _get_first_path(root)
    assert path is not None
    assert '#fff' == path.attrib.get('fill')
    assert path.attrib.get('class') is None
    d = path.attrib.get('d')
    assert d
    expected = 'M0 0h{1}v{0}h-{1}z'.format(*qr.symbol_size())
    assert expected == d
    g = _get_group(root)
    assert g is None


def test_scale_background():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark='green', light='yellow', scale=10)
    root = _parse_xml(svg)
    g = _get_group(root)
    assert g is not None
    assert 'scale(10)' == g.attrib.get('transform')


def test_write_svg_color_rgb():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark=(76, 131, 205))
    assert svg.startswith('<svg')
    root = _parse_xml(svg)
    assert 'viewBox' not in root.attrib
    assert 'height' in root.attrib
    assert 'width' in root.attrib
    css_class = root.attrib.get('class')
    assert css_class
    assert _CSS_CLASS == css_class
    path_el = _get_first_path(root)
    assert path_el is not None
    path_class = path_el.get('class')
    assert _PATH_CLASS == path_class
    stroke = path_el.get('stroke')
    assert stroke == '#4c83cd'
    title_el = _get_title(root)
    assert title_el is None
    desc_el = _get_desc(root)
    assert desc_el is None


def test_write_svg_color_rgba():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark='#0000ffcc')
    assert 'stroke-opacity' in svg


def test_write_svg_color_rgba_svg2():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark='#0000ffcc', svgversion=2.0)
    assert 'stroke-opacity' not in svg
    root = _parse_xml(svg)
    path = _get_first_path(root)
    assert path.attrib['stroke'].startswith('rgba')


def test_write_svg_background_rgba():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(light='#0000ffcc')
    assert 'fill-opacity' in svg


def test_write_svg_background_rgba_svg2():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(light='#0000ffcc', svgversion=2.0)
    assert 'fill-opacity' not in svg
    root = _parse_xml(svg)
    path = _get_first_path(root)
    assert path.attrib['fill'].startswith('rgba')


def test_write_no_xmldecl():
    qr = segno.make_qr('test')
    with pytest.raises(TypeError):
        qr.svg_inline(xmldecl=False)


def test_viewbox():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(omitsize=True)
    root = _parse_xml(svg)
    assert 'viewBox' in root.attrib
    assert 'height' not in root.attrib
    assert 'width' not in root.attrib


def test_svgid():
    qr = segno.make_qr('test')
    ident = 'svgid'
    svg = qr.svg_inline(svgid=ident)
    root = _parse_xml(svg)
    assert 'id' in root.attrib
    assert ident == root.attrib['id']


def test_svgid_default():
    qr = segno.make_qr('test')
    svg = qr.svg_inline()
    root = _parse_xml(svg)
    assert 'id' not in root.attrib


def test_svgid_empty_string():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(svgid='')
    root = _parse_xml(svg)
    assert 'id' not in root.attrib


def test_svgversion():
    qr = segno.make_qr('test')
    version = 1.0
    svg = qr.svg_inline(svgversion=version)
    root = _parse_xml(svg)
    assert 'version' in root.attrib
    assert str(version) == root.attrib['version']


def test_svgversion_default():
    qr = segno.make_qr('test')
    svg = qr.svg_inline()
    root = _parse_xml(svg)
    assert 'version' not in root.attrib


def test_no_svg_class():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(svgclass=None)
    root = _parse_xml(svg)
    assert 'class' not in root.attrib


def test_no_svg_class_empty_str():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(svgclass='')
    root = _parse_xml(svg)
    assert 'class' not in root.attrib


def test_custom_svg_class():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(svgclass='test-class')
    root = _parse_xml(svg)
    assert 'class' in root.attrib
    assert 'test-class' == root.attrib.get('class')


def test_no_line_class():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(lineclass=None)
    root = _parse_xml(svg)
    path_el = _get_first_path(root)
    assert 'class' not in path_el.attrib


def test_no_line_class_empty_str():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(lineclass='')
    root = _parse_xml(svg)
    path_el = _get_first_path(root)
    assert 'class' not in path_el.attrib


def test_custom_line_class():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(lineclass='test-class')
    root = _parse_xml(svg)
    path_el = _get_first_path(root)
    assert 'class' in path_el.attrib
    assert 'test-class' == path_el.attrib.get('class')


def test_title():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(title='Test')
    root = _parse_xml(svg)
    title_el = _get_title(root)
    assert title_el is not None
    assert 'Test' == title_el.text


def test_title2():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(title='Määhhh')
    root = _parse_xml(svg)
    title_el = _get_title(root)
    assert title_el is not None
    assert 'Määhhh' == title_el.text


def test_title3():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(title='点')
    root = _parse_xml(svg)
    title_el = _get_title(root)
    assert title_el is not None
    assert '点' == title_el.text


def test_title4():
    qr = segno.make_qr('test')
    encoding = 'ISO-8859-1'
    svg = qr.svg_inline(title='áà', encoding=encoding)
    root = _parse_xml(svg)
    title_el = _get_title(root)
    assert title_el is not None
    assert 'áà' == title_el.text


def test_title_escape():
    qr = segno.make_qr('test')
    title = '<title>&</title>'
    svg = qr.svg_inline(title=title)
    assert '<title>&lt;title&gt;&amp;&lt;/title&gt;</title>' in svg
    root = _parse_xml(svg)
    title_el = _get_title(root)
    assert title_el is not None
    assert title == title_el.text


def test_desc():
    qr = segno.make_qr('test')
    desc = 'Test'
    svg = qr.svg_inline(desc=desc)
    root = _parse_xml(svg)
    desc_el = _get_desc(root)
    assert desc_el is not None
    assert desc == desc_el.text


def test_desc2():
    qr = segno.make_qr('test')
    desc = 'Määhhhh'
    svg = qr.svg_inline(desc=desc)
    root = _parse_xml(svg)
    desc_el = _get_desc(root)
    assert desc_el is not None
    assert desc == desc_el.text


def test_desc3():
    qr = segno.make_qr('test')
    desc = '点'
    svg = qr.svg_inline(desc=desc)
    root = _parse_xml(svg)
    desc_el = _get_desc(root)
    assert desc_el is not None
    assert desc == desc_el.text


def test_desc4():
    qr = segno.make_qr('test')
    encoding = 'ISO-8859-1'
    desc = 'áà'
    svg = qr.svg_inline(desc='áà', encoding=encoding)
    root = _parse_xml(svg)
    desc_el = _get_desc(root)
    assert desc_el is not None
    assert desc == desc_el.text


def test_desc_escape():
    qr = segno.make_qr('test')
    desc = '<desc>&</desc>'
    svg = qr.svg_inline(desc=desc)
    assert '<desc>&lt;desc&gt;&amp;&lt;/desc&gt;</desc>' in svg
    root = _parse_xml(svg)
    desc_el = _get_desc(root)
    assert desc_el is not None
    assert desc == desc_el.text


def test_background():
    qr = segno.make_qr('test')
    color = '#800080'
    svg = qr.svg_inline(light=color)
    root = _parse_xml(svg)
    # Background should be the first path in the doc
    rect = _get_first_path(root)
    assert rect is not None
    assert color == rect.attrib['fill']


def test_module_color():
    qr = segno.make_qr('test')
    color = '#800080'
    svg = qr.svg_inline(dark=color)
    root = _parse_xml(svg)
    path = _get_first_path(root)
    assert path is not None
    assert color == path.attrib['stroke']


def test_scale():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(scale=2)
    root = _parse_xml(svg)
    path = _get_first_path(root)
    assert path is not None
    assert 'scale(2)' in path.attrib['transform']


def test_scale_float():
    qr = segno.make_qr('test')
    scale = 2.13
    svg = qr.svg_inline(scale=scale)
    root = _parse_xml(svg)
    path = _get_first_path(root)
    assert path is not None
    assert 'scale({0})'.format(scale) in path.attrib['transform']


def test_unit_omitsize():
    qr = segno.make_qr('test')
    with pytest.raises(ValueError):
        qr.svg_inline(unit='cm', omitsize=True)


def test_unit():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(unit='mm')
    width, height = qr.symbol_size()
    root = _parse_xml(svg)
    assert 'width' in root.attrib
    assert '%dmm' % width == root.attrib['width']
    assert 'height' in root.attrib
    assert '%dmm' % height == root.attrib['height']
    assert 'viewBox' in root.attrib
    assert '0 0 %d %d' % (width, height) == root.attrib['viewBox']


def test_unit_none():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(unit=None)
    width, height = qr.symbol_size()
    root = _parse_xml(svg)
    assert 'width' in root.attrib
    assert str(width) == root.attrib['width']
    assert 'height' in root.attrib
    assert str(height) == root.attrib['height']
    assert 'viewBox' not in root.attrib


def test_draw_transparent():
    qr = segno.make_qr('test')
    svg = qr.svg_inline(dark='green', finder_dark='green', dark_module='blue',
                        alignment_light='yellow', quiet_zone='yellow',
                        draw_transparent=False)
    root = _parse_xml(svg)
    paths = root.findall('.//path')
    assert 3 == len(paths)
    svg = qr.svg_inline(dark='green', finder_dark='green', dark_module='blue',
                        alignment_light='yellow', quiet_zone='yellow',
                        draw_transparent=True)
    root = _parse_xml(svg)
    paths = root.findall('.//path')
    assert 4 == len(paths)
    assert 1 == len([p for p in paths if p.attrib.get('stroke') is None])


if __name__ == '__main__':
    pytest.main([__file__])



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 - 2020 -- Lars Heuer
# All rights reserved.
#
# License: BSD License
#
"""\
Tests against the ``utils.matrix_iter_verboses`` function.
"""
from __future__ import absolute_import, unicode_literals, print_function
import io
import os
import pytest
from segno import encoder, utils, consts


def read_matrix(name):
    """\
    Helper function to read a matrix from /ref_matrix. The file extension .txt
    is added automatically.

    :return: A tuple of bytearrays
    """
    matrix = []
    with io.open(os.path.join(os.path.dirname(__file__),
                              'feature_decompose/{0}.txt'.format(name)), 'rt') as f:
        for row in f:
            matrix.append(bytearray([int(i) for i in row if i != '\n']))
    return matrix


def test_finder_pattern_dark_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_FINDER_PATTERN_DARK] for v in row]))
    expected = read_matrix('v1-finder-dark')
    assert expected == res


def test_finder_pattern_dark_light_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_FINDER_PATTERN_LIGHT] for v in row]))
    expected = read_matrix('v1-finder-light')
    assert expected == res


def test_finder_pattern_dark_and_light_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_FINDER_PATTERN_DARK,
                                               consts.TYPE_FINDER_PATTERN_LIGHT)] for v in row]))  # noqa: E501
    expected = read_matrix('v1-finder-dark-and-light')
    assert expected == res


def test_finder_pattern_dark_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_FINDER_PATTERN_DARK] for v in row]))
    expected = read_matrix('m2-finder-dark')
    assert expected == res


def test_finder_pattern_dark_light_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_FINDER_PATTERN_LIGHT] for v in row]))
    expected = read_matrix('m2-finder-light')
    assert expected == res


def test_finder_pattern_dark_and_light_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_FINDER_PATTERN_DARK,
                                               consts.TYPE_FINDER_PATTERN_LIGHT)] for v in row]))  # noqa: E501
    expected = read_matrix('m2-finder-dark-and-light')
    assert expected == res


def test_separator_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_SEPARATOR] for v in row]))
    expected = read_matrix('v1-separator')
    assert expected == res


def test_separator_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_SEPARATOR] for v in row]))
    expected = read_matrix('m2-separator')
    assert expected == res


def test_darkmodule_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_DARKMODULE] for v in row]))
    expected = read_matrix('v1-darkmodule')
    assert expected == res


def test_no_darkmodule_mqr():
    # Micro QR Codes don't have a dark module.
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.extend([v == consts.TYPE_DARKMODULE for v in row])
    assert True not in res


def test_timing_dark_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_TIMING_DARK] for v in row]))
    expected = read_matrix('v1-timing-dark')
    assert expected == res


def test_timing_light_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_TIMING_LIGHT] for v in row]))
    expected = read_matrix('v1-timing-light')
    assert expected == res


def test_timing_dark_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_TIMING_DARK] for v in row]))
    expected = read_matrix('m2-timing-dark')
    assert expected == res


def test_timing_light_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_TIMING_LIGHT] for v in row]))
    expected = read_matrix('m2-timing-light')
    assert expected == res


def test_timing_dark_and_light_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_TIMING_DARK,
                                               consts.TYPE_TIMING_LIGHT)] for v in row]))
    expected = read_matrix('m2-timing-dark-and-light')
    assert expected == res


def test_alignment_dark():
    qr = encoder.encode('A', version=12)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_ALIGNMENT_PATTERN_DARK] for v in row]))
    expected = read_matrix('v12-alignment-dark')
    assert expected == res


def test_alignment_light():
    qr = encoder.encode('A', version=12)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_ALIGNMENT_PATTERN_LIGHT] for v in row]))
    expected = read_matrix('v12-alignment-light')
    assert expected == res


def test_alignment_dark_and_light():
    qr = encoder.encode('A', version=12)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_ALIGNMENT_PATTERN_LIGHT,
                                               consts.TYPE_ALIGNMENT_PATTERN_DARK)] for v in row]))  # noqa: E501
    expected = read_matrix('v12-alignment-dark-and-light')
    assert expected == res


def test_version_dark():
    qr = encoder.encode('A', version=7)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_VERSION_DARK] for v in row]))
    expected = read_matrix('v7-version-dark')
    assert expected == res


def test_version_light():
    qr = encoder.encode('A', version=7)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_VERSION_LIGHT] for v in row]))
    expected = read_matrix('v7-version-light')
    assert expected == res


def test_version_dark_and_light():
    qr = encoder.encode('A', version=7)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_VERSION_LIGHT,
                                               consts.TYPE_VERSION_DARK)] for v in row]))  # noqa: E501
    expected = read_matrix('v7-version-dark-and-light')
    assert expected == res


def test_version_no_version():
    # The version information is not available in QR Codes < 7
    qr = encoder.encode('A', version=6)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.extend([v in (consts.TYPE_VERSION_LIGHT, consts.TYPE_VERSION_DARK) for v in row])
    assert True not in res


def test_format_dark_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v == consts.TYPE_FORMAT_DARK] for v in row]))
    expected = read_matrix('v1-format-dark')
    assert expected == res


def test_format_light_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_FORMAT_LIGHT] for v in row]))
    expected = read_matrix('v1-format-light')
    assert expected == res


def test_format_dark_and_light_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=0):
        res.append(bytearray([(0x2, 0x1)[v in (consts.TYPE_FORMAT_DARK,
                                               consts.TYPE_FORMAT_LIGHT)] for v in row]))
    expected = read_matrix('v1-format-dark-and-light')
    assert expected == res


def test_quietzone_default_qr():
    qr = encoder.encode('A', micro=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_QUIET_ZONE] for v in row]))
    expected = read_matrix('v1-quietzone-4')
    assert expected == res


def test_quietzone_custom_qr():
    qr = encoder.encode('A', micro=False)
    border = 1
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=border):
        res.append(bytearray([(0x2, 0x0)[v == consts.TYPE_QUIET_ZONE] for v in row]))
    expected = read_matrix('v1-quietzone-1')
    assert expected == res


def test_quietzone_default_mqr():
    qr = encoder.encode('A', micro=True)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version):
        res.append(bytearray([(0x1, 0x0)[v == consts.TYPE_QUIET_ZONE] for v in row]))
    expected = read_matrix('m2-quietzone-2')
    assert expected == res


def test_quietzone_custom_mqr():
    qr = encoder.encode('A', micro=True)
    border = 5
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version, border=border):
        res.append(bytearray([(0x1, 0x0)[v == consts.TYPE_QUIET_ZONE] for v in row]))
    expected = read_matrix('m2-quietzone-5')
    assert expected == res


def test_convert_to_boolean_true():
    qr = encoder.encode('ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                        error='m', mask=4, boost_error=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version):
        res.append(bytearray([int(v >> 8 > 0) for v in row]))
    expected = read_matrix('iso-fig-29')
    assert expected == res


def test_convert_to_boolean_false():
    qr = encoder.encode('ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                        error='m', mask=4, boost_error=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version):
        res.append(bytearray([not int(v >> 8 == 0) for v in row]))
    expected = read_matrix('iso-fig-29')
    assert expected == res


def test_convert_to_boolean():
    qr = encoder.encode('ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                        error='m', mask=4, boost_error=False)
    res = []
    for row in utils.matrix_iter_verbose(qr.matrix, qr.version):
        res.append(bytearray([bool(v >> 8) for v in row]))
    expected = read_matrix('iso-fig-29')
    assert expected == res


if __name__ == '__main__':
    pytest.main([__file__])
